from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes, ConvertToDlString
from RSTAB.enums import MemberLoadDirection, MemberLoadType, MemberLoadDistribution, MemberLoadDirection, MemberLoadDirectionOrientation
from RSTAB.enums import MemberLoadEccentricityHorizontalAlignment, MemberLoadEccentricityVerticalAlignment, MemberLoadEccentricitySectionMiddle
from RSTAB.enums import MemberLoadAxisDefinitionType, MemberLoadAxisDefinitionAxisOrientation, MemberLoadAxisDefinition

class MemberLoad():

    def __init__(self,
                 no: int = 1,
                 load_case_no: int = 1,
                 members_no: str = '1',
                 load_direction = MemberLoadDirection.LOAD_DIRECTION_LOCAL_Z,
                 magnitude: float = 2000,
                 comment: str = '',
                 params: dict = None,
                 model = Model):
        """
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            members_no (str): Assigned Members
            load_direction (enum): Load Directin Enumeration
            magnitude (float): Load Magnitude
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """
        # Client model | Member Load
        clientObject = model.clientModel.factory.create('ns0:member_load')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Load No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Members No. (e.g. '5 6 7 12')
        clientObject.members = ConvertToDlString(members_no)

        # Member Load Type
        load_type = MemberLoadType.LOAD_TYPE_FORCE
        clientObject.load_type = load_type.name

        # Member Load Distribution
        load_distribution = MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM
        clientObject.load_distribution = load_distribution.name

        # Member Load Direction
        clientObject.load_direction = load_direction.name

        # Load Magnitude
        clientObject.magnitude = magnitude

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Load Member Load to client model
        model.clientModel.service.set_member_load(load_case_no, clientObject)

    @staticmethod
    def Force(
                 no: int = 1,
                 load_case_no: int = 1,
                 members_no: str = '1',
                 load_distribution= MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM,
                 load_direction= MemberLoadDirection.LOAD_DIRECTION_LOCAL_Z,
                 load_parameter: list = None,
                 force_eccentricity: bool= False,
                 list_reference: bool= False,
                 comment: str = '',
                 params: dict = None,
                 model = Model):
        """
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            members_no (str): Assigned Member(s)
            load_distribution (enum): Member Load Distribution Enumeration
            load_direction (enum): Member Load Direction Enumeration
            load_parameter (list/list of lists): Load Parameter List
                for load_distribution == LOAD_DISTRIBUTION_UNIFORM:
                    load_parameter = [magnitude]
                for load_distribution == LOAD_DISTRIBUTION_UNIFORM_TOTAL:
                    load_parameter = [magnitude]
                for load_distribution == LOAD_DISTRIBUTION_CONCENTRATED_1:
                    load_parameter = [relative_distance = False, magnitude, distance_a]
                for load_distribution == LOAD_DISTRIBUTION_CONCENTRATED_N:
                    load_parameter = [relative_distance_a = False, relative_distance_b = False, magnitude, count_n, distance_a, distance_b]
                for load_distribution == LOAD_DISTRIBUTION_CONCENTRATED_2x2:
                    load_parameter = [relative_distance_a = False, relative_distance_b = False, relative_distance_c = False, magnitude, distance_a, distance_b, distance_c]
                for load_distribution == LOAD_DISTRIBUTION_CONCENTRATED_2:
                    load_parameter = [relative_distance_a = False, relative_distance_b = False, magnitude_1, magnitude_2, distance_a, distance_b]
                for load_distribution == LOAD_DISTRIBUTION_CONCENTRATED_VARYING:
                    load_parameter = [[distance, magnitude], ...]
                for load_distribution == LOAD_DISTRIBUTION_TRAPEZOIDAL:
                    load_parameter = [relative_distance_a = False, relative_distance_b = False,magnitude_1, magnitude_2, distance_a, distance_b]
                for load_distribution == LOAD_DISTRIBUTION_TAPERED:
                    load_parameter = [relative_distance_a = False, relative_distance_b = False,magnitude_1, magnitude_2, distance_a, distance_b]
                for load_distribution == LOAD_DISTRIBUTION_PARABOLIC:
                    load_parameter = [magnitude_1, magnitude_2, magnitude_3]
                for load_distribution == LOAD_DISTRIBUTION_VARYING:
                    load_parameter = [[distance, magnitude], ...]
                for load_distribution == LOAD_DISTRIBUTION_VARYING_IN_Z:
                    load_parameter = [[distance, magnitude], ...]
            force_eccentricity (bool): Enable/Disable Force Eccentricity Option
            list_reference (bool): Enable/Disable List Reference Option
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
                for force_eccentricity == True:
                    params = {'eccentricity_horizontal_alignment': MemberLoadEccentricityHorizontalAlignment.ALIGN_NONE,
                        'eccentricity_vertical_alignment': MemberLoadEccentricityVerticalAlignment.ALIGN_NONE,
                        'eccentricity_section_middle': MemberLoadEccentricitySectionMiddle.LOAD_ECCENTRICITY_SECTION_MIDDLE_CENTER_OF_GRAVITY,
                        'is_eccentricity_at_end_different_from_start': False,
                        'eccentricity_y_at_end': 0.0,
                        'eccentricity_y_at_start': 0.0,
                        'eccentricity_z_at_end': 0.0,
                        'eccentricity_z_at_start': 0.0}
            model (RSTAB Class, optional): Model to be edited
        """
        # Client model | Member Load
        clientObject = model.clientModel.factory.create('ns0:member_load')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Load No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Members No. (e.g. '5 6 7 12')
        clientObject.members = ConvertToDlString(members_no)

        # Member Load Type
        load_type = MemberLoadType.LOAD_TYPE_FORCE
        clientObject.load_type = load_type.name

        # Member Load Distribution
        clientObject.load_distribution= load_distribution.name

        # Load Magnitude and Parameters
        if load_distribution.name == "LOAD_DISTRIBUTION_UNIFORM" or load_distribution.name == "LOAD_DISTRIBUTION_UNIFORM_TOTAL":
            clientObject.magnitude = load_parameter[0]

        elif load_distribution.name == "LOAD_DISTRIBUTION_CONCENTRATED_1":
            clientObject.distance_a_is_defined_as_relative = load_parameter[0]
            if load_parameter[0] == False:
                clientObject.magnitude = load_parameter[1]
                clientObject.distance_a_absolute = load_parameter[2]
            else:
                clientObject.magnitude = load_parameter[1]
                clientObject.distance_a_relative = load_parameter[2]

        elif load_distribution.name ==  "LOAD_DISTRIBUTION_CONCENTRATED_N":
            clientObject.distance_a_is_defined_as_relative = load_parameter[0]
            clientObject.distance_b_is_defined_as_relative = load_parameter[1]
            clientObject.magnitude = load_parameter[2]
            clientObject.count_n = load_parameter[3]

            if load_parameter[0] == False:
                clientObject.distance_a_absolute = load_parameter[4]
            else:
                clientObject.distance_a_relative = load_parameter[4]

            if load_parameter[1] == False:
                clientObject.distance_b_absolute = load_parameter[5]
            else:
                clientObject.distance_b_relative = load_parameter[5]

        elif load_distribution.name == "LOAD_DISTRIBUTION_CONCENTRATED_2x2":
            clientObject.distance_a_is_defined_as_relative = load_parameter[0]
            clientObject.distance_b_is_defined_as_relative = load_parameter[1]
            clientObject.distance_c_is_defined_as_relative = load_parameter[2]
            clientObject.magnitude = load_parameter[3]

            if load_parameter[0] == False:
                clientObject.distance_a_absolute = load_parameter[4]
            else:
                clientObject.distance_a_relative = load_parameter[4]

            if load_parameter[1] == False:
                clientObject.distance_b_absolute = load_parameter[5]
            else:
                clientObject.distance_b_relative = load_parameter[5]

            if load_parameter[2] == False:
                clientObject.distance_c_absolute = load_parameter[6]
            else:
                clientObject.distance_c_relative = load_parameter[6]

        elif load_distribution.name == "LOAD_DISTRIBUTION_CONCENTRATED_2":
            clientObject.distance_a_is_defined_as_relative = load_parameter[0]
            clientObject.distance_b_is_defined_as_relative = load_parameter[1]
            clientObject.magnitude_1 = load_parameter[2]
            clientObject.magnitude_2 = load_parameter[3]

            if load_parameter[0] == False:
                clientObject.distance_a_absolute = load_parameter[4]
            else:
                clientObject.distance_a_relative = load_parameter[4]

            if load_parameter[1] == False:
                clientObject.distance_b_absolute = load_parameter[5]
            else:
                clientObject.distance_b_relative = load_parameter[5]

        elif load_distribution.name == "LOAD_DISTRIBUTION_CONCENTRATED_VARYING":
            clientObject.varying_load_parameters = model.clientModel.factory.create('ns0:member_load.varying_load_parameters')

            for i,j in enumerate(load_parameter):
                if len(load_parameter[i]) != 2:
                    raise ValueError("WARNING: MemberLoad no: %x, load case: %x - Wrong data input." % (no, load_case_no))
                mlvlp = model.clientModel.factory.create('ns0:member_load_varying_load_parameters_row')
                mlvlp.no = i+1
                mlvlp.row = model.clientModel.factory.create('ns0:member_load_varying_load_parameters')
                clearAttributes(mlvlp.row)
                mlvlp.row.distance = load_parameter[i][0]
                mlvlp.row.magnitude = load_parameter[i][1]

                clientObject.varying_load_parameters.member_load_varying_load_parameters.append(mlvlp)

        elif load_distribution.name == "LOAD_DISTRIBUTION_TRAPEZOIDAL":
            clientObject.distance_a_is_defined_as_relative = load_parameter[0]
            clientObject.distance_b_is_defined_as_relative = load_parameter[1]
            clientObject.magnitude_1 = load_parameter[2]
            clientObject.magnitude_2 = load_parameter[3]

            if load_parameter[0] == False:
                clientObject.distance_a_absolute = load_parameter[4]
            else:
                clientObject.distance_a_relative = load_parameter[4]

            if load_parameter[1] == False:
                clientObject.distance_b_absolute = load_parameter[5]
            else:
                clientObject.distance_b_relative = load_parameter[5]

        elif load_distribution.name == "LOAD_DISTRIBUTION_TAPERED":
            clientObject.distance_a_is_defined_as_relative = load_parameter[0]
            clientObject.distance_b_is_defined_as_relative = load_parameter[1]
            clientObject.magnitude_1 = load_parameter[2]
            clientObject.magnitude_2 = load_parameter[3]

            if load_parameter[0] == False:
                clientObject.distance_a_absolute = load_parameter[4]
            else:
                clientObject.distance_a_relative = load_parameter[4]

            if load_parameter[1] == False:
                clientObject.distance_b_absolute = load_parameter[5]
            else:
                clientObject.distance_b_relative = load_parameter[5]

        elif load_distribution.name == "LOAD_DISTRIBUTION_PARABOLIC":
            clientObject.magnitude_1 = load_parameter[0]
            clientObject.magnitude_2 = load_parameter[1]
            clientObject.magnitude_3 = load_parameter[2]

        elif load_distribution.name == "LOAD_DISTRIBUTION_VARYING":
            clientObject.varying_load_parameters = model.clientModel.factory.create('ns0:member_load.varying_load_parameters')
            for i,j in enumerate(load_parameter):
                if len(load_parameter[i]) != 2:
                    raise ValueError("WARNING: MemberLoad no: %x, load case: %x - Wrong data input." % (no, load_case_no))
                mlvlp = model.clientModel.factory.create('ns0:member_load_varying_load_parameters_row')
                mlvlp.no = i+1
                mlvlp.row = model.clientModel.factory.create('ns0:member_load_varying_load_parameters')
                clearAttributes(mlvlp.row)
                mlvlp.row.distance = load_parameter[i][0]
                mlvlp.row.magnitude = load_parameter[i][1]

                clientObject.varying_load_parameters.member_load_varying_load_parameters.append(mlvlp)

        elif load_distribution.name == "LOAD_DISTRIBUTION_VARYING_IN_Z":
            clientObject.varying_load_parameters = model.clientModel.factory.create('ns0:member_load.varying_load_parameters')
            for i,j in enumerate(load_parameter):
                if len(load_parameter[i]) != 2:
                    raise ValueError("WARNING: MemberLoad no: %x, load case: %x - Wrong data input." % (no, load_case_no))
                mlvlp = model.clientModel.factory.create('ns0:member_load_varying_load_parameters_row')
                mlvlp.no = i+1
                mlvlp.row = model.clientModel.factory.create('ns0:member_load_varying_load_parameters')
                clearAttributes(mlvlp.row)
                mlvlp.row.distance = load_parameter[i][0]
                mlvlp.row.magnitude = load_parameter[i][1]

                clientObject.varying_load_parameters.member_load_varying_load_parameters.append(mlvlp)

        # Member Load Direction
        clientObject.load_direction = load_direction.name

        # Reference to List of Members
        clientObject.reference_to_list_of_members = list_reference

        # Force Eccentiricity
        clientObject.has_force_eccentricity = force_eccentricity

        if force_eccentricity == True:

            params_ecc = {'eccentricity_horizontal_alignment': MemberLoadEccentricityHorizontalAlignment.ALIGN_NONE,
                           'eccentricity_vertical_alignment': MemberLoadEccentricityVerticalAlignment.ALIGN_NONE,
                           'eccentricity_section_middle': MemberLoadEccentricitySectionMiddle.LOAD_ECCENTRICITY_SECTION_MIDDLE_CENTER_OF_GRAVITY,
                           'is_eccentricity_at_end_different_from_start': False,
                           'eccentricity_y_at_end': 0.0,
                           'eccentricity_y_at_start': 0.0,
                           'eccentricity_z_at_end': 0.0,
                           'eccentricity_z_at_start': 0.0}

            params_ecc.update(params)

            if params_ecc['is_eccentricity_at_end_different_from_start'] == False:

                clientObject.eccentricity_horizontal_alignment= params_ecc['eccentricity_horizontal_alignment'].name
                clientObject.eccentricity_vertical_alignment= params_ecc['eccentricity_vertical_alignment'].name
                clientObject.eccentricity_section_middle = params_ecc['eccentricity_section_middle'].name
                clientObject.eccentricity_y_at_end= params_ecc['eccentricity_y_at_start']
                clientObject.eccentricity_y_at_start= params_ecc['eccentricity_y_at_start']
                clientObject.eccentricity_z_at_end= params_ecc['eccentricity_z_at_start']
                clientObject.eccentricity_z_at_start= params_ecc['eccentricity_z_at_start']

            elif params_ecc['is_eccentricity_at_end_different_from_start'] == True:

                clientObject.eccentricity_horizontal_alignment= params_ecc['eccentricity_horizontal_alignment']
                clientObject.eccentricity_vertical_alignment= params_ecc['eccentricity_vertical_alignment']
                clientObject.eccentricity_section_middle = params_ecc['eccentricity_section_middle']
                clientObject.eccentricity_y_at_end= params_ecc['eccentricity_y_at_end']
                clientObject.eccentricity_y_at_start= params_ecc['eccentricity_y_at_start']
                clientObject.eccentricity_z_at_end= params_ecc['eccentricity_z_at_end']
                clientObject.eccentricity_z_at_start= params_ecc['eccentricity_z_at_start']

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if 'eccentricity_horizontal_alignment' or 'eccentricity_vertical_alignment' or 'eccentricity_section_middle' or 'is_eccentricity_at_end_different_from_start' or 'eccentricity_y_at_end' or 'eccentricity_y_at_start' or 'eccentricity_z_at_end' or 'eccentricity_z_at_start':
            pass
        else:
            if params:
                for key in params:
                    clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Load Member Load to client model
        model.clientModel.service.set_member_load(load_case_no, clientObject)

    @staticmethod
    def Moment(
                 no: int = 1,
                 load_case_no: int = 1,
                 members_no: str = '1',
                 load_distribution= MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM,
                 load_direction= MemberLoadDirection.LOAD_DIRECTION_LOCAL_Z,
                 load_parameter: list = None,
                 list_reference: bool= False,
                 comment: str = '',
                 params: dict = None,
                 model = Model):
        """
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            members_no (str): Assigned Member(s)
            load_distribution (enum): Load Distribution Enumeration
            load_direction (enum): Load Direction Enumeration
            load_parameter (list/list of lists): Load Parameter List
                for load_distribution == LOAD_DISTRIBUTION_UNIFORM:
                    load_parameter = [magnitude]
                for load_distribution == LOAD_DISTRIBUTION_CONCENTRATED_1:
                    load_parameter = [relative_distance = False, magnitude, distance_a]
                for load_distribution == LOAD_DISTRIBUTION_CONCENTRATED_N:
                    load_parameter = [relative_distance_a = False, relative_distance_b = False, magnitude, count_n, distance_a, distance_b]
                for load_distribution == LOAD_DISTRIBUTION_CONCENTRATED_2x2:
                    load_parameter = [relative_distance_a = False, relative_distance_b = False, relative_distance_c = False, magnitude, distance_a, distance_b, distance_c]
                for load_distribution == LOAD_DISTRIBUTION_CONCENTRATED_2:
                    load_parameter = [relative_distance_a = False, relative_distance_b = False, magnitude_1, magnitude_2, distance_a, distance_b]
                for load_distribution == LOAD_DISTRIBUTION_CONCENTRATED_VARYING:
                    load_parameter = [[distance, magnitude], ...]
                for load_distribution == LOAD_DISTRIBUTION_TRAPEZOIDAL:
                    load_parameter = [relative_distance_a = False, relative_distance_b = False,magnitude_1, magnitude_2, distance_a, distance_b]
                for load_distribution == LOAD_DISTRIBUTION_TAPERED:
                    load_parameter = [relative_distance_a = False, relative_distance_b = False,magnitude_1, magnitude_2, distance_a, distance_b]
                for load_distribution == LOAD_DISTRIBUTION_PARABOLIC:
                    load_parameter = [magnitude_1, magnitude_2, magnitude_3]
                for load_distribution == LOAD_DISTRIBUTION_VARYING:
                    load_parameter = [[distance, magnitude], ...]
            list_reference (bool): Enable/Disable List Reference Option
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """
        # Client model | Member Load
        clientObject = model.clientModel.factory.create('ns0:member_load')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Load No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Members No. (e.g. '5 6 7 12')
        clientObject.members = ConvertToDlString(members_no)

        # Member Load Type
        load_type = MemberLoadType.LOAD_TYPE_MOMENT
        clientObject.load_type = load_type.name

        # Member Load Distribution
        clientObject.load_distribution= load_distribution.name

        # Load Magnitude and Parameters
        if load_distribution.name == "LOAD_DISTRIBUTION_UNIFORM":
            clientObject.magnitude = load_parameter[0]

        elif load_distribution.name == "LOAD_DISTRIBUTION_CONCENTRATED_1":
            clientObject.distance_a_is_defined_as_relative = load_parameter[0]
            if load_parameter[0] == False:
                clientObject.magnitude = load_parameter[1]
                clientObject.distance_a_absolute = load_parameter[2]
            else:
                clientObject.magnitude = load_parameter[1]
                clientObject.distance_a_relative = load_parameter[2]

        elif load_distribution.name ==  "LOAD_DISTRIBUTION_CONCENTRATED_N":
            clientObject.distance_a_is_defined_as_relative = load_parameter[0]
            clientObject.distance_b_is_defined_as_relative = load_parameter[1]
            clientObject.magnitude = load_parameter[2]
            clientObject.count_n = load_parameter[3]

            if load_parameter[0] == False:
                clientObject.distance_a_absolute = load_parameter[4]
            else:
                clientObject.distance_a_relative = load_parameter[4]

            if load_parameter[1] == False:
                clientObject.distance_b_absolute = load_parameter[5]
            else:
                clientObject.distance_b_relative = load_parameter[5]

        elif load_distribution.name == "LOAD_DISTRIBUTION_CONCENTRATED_2x2":
            clientObject.distance_a_is_defined_as_relative = load_parameter[0]
            clientObject.distance_b_is_defined_as_relative = load_parameter[1]
            clientObject.distance_c_is_defined_as_relative = load_parameter[2]
            clientObject.magnitude = load_parameter[3]

            if load_parameter[0] == False:
                clientObject.distance_a_absolute = load_parameter[4]
            else:
                clientObject.distance_a_relative = load_parameter[4]

            if load_parameter[1] == False:
                clientObject.distance_b_absolute = load_parameter[5]
            else:
                clientObject.distance_b_relative = load_parameter[5]

            if load_parameter[2] == False:
                clientObject.distance_c_absolute = load_parameter[6]
            else:
                clientObject.distance_c_relative = load_parameter[6]

        elif load_distribution.name == "LOAD_DISTRIBUTION_CONCENTRATED_2":
            clientObject.distance_a_is_defined_as_relative = load_parameter[0]
            clientObject.distance_b_is_defined_as_relative = load_parameter[1]
            clientObject.magnitude_1 = load_parameter[2]
            clientObject.magnitude_2 = load_parameter[3]

            if load_parameter[0] == False:
                clientObject.distance_a_absolute = load_parameter[4]
            else:
                clientObject.distance_a_relative = load_parameter[4]

            if load_parameter[1] == False:
                clientObject.distance_b_absolute = load_parameter[5]
            else:
                clientObject.distance_b_relative = load_parameter[5]

        elif load_distribution.name == "LOAD_DISTRIBUTION_CONCENTRATED_VARYING":
            clientObject.varying_load_parameters = model.clientModel.factory.create('ns0:member_load.varying_load_parameters')

            for i,j in enumerate(load_parameter):
                if len(load_parameter[i]) != 2:
                    raise ValueError("WARNING: MemberLoad no: %x, load case: %x - Wrong data input." % (no, load_case_no))
                mlvlp = model.clientModel.factory.create('ns0:member_load_varying_load_parameters_row')
                mlvlp.no = i+1
                mlvlp.row = model.clientModel.factory.create('ns0:member_load_varying_load_parameters')
                clearAttributes(mlvlp.row)
                mlvlp.row.distance = load_parameter[i][0]
                mlvlp.row.magnitude = load_parameter[i][1]

                clientObject.varying_load_parameters.member_load_varying_load_parameters.append(mlvlp)

        elif load_distribution.name == "LOAD_DISTRIBUTION_TRAPEZOIDAL":
            clientObject.distance_a_is_defined_as_relative = load_parameter[0]
            clientObject.distance_b_is_defined_as_relative = load_parameter[1]
            clientObject.magnitude_1 = load_parameter[2]
            clientObject.magnitude_2 = load_parameter[3]

            if load_parameter[0] == False:
                clientObject.distance_a_absolute = load_parameter[4]
            else:
                clientObject.distance_a_relative = load_parameter[4]

            if load_parameter[1] == False:
                clientObject.distance_b_absolute = load_parameter[5]
            else:
                clientObject.distance_b_relative = load_parameter[5]

        elif load_distribution.name == "LOAD_DISTRIBUTION_TAPERED":
            clientObject.distance_a_is_defined_as_relative = load_parameter[0]
            clientObject.distance_b_is_defined_as_relative = load_parameter[1]
            clientObject.magnitude_1 = load_parameter[2]
            clientObject.magnitude_2 = load_parameter[3]

            if load_parameter[0] == False:
                clientObject.distance_a_absolute = load_parameter[4]
            else:
                clientObject.distance_a_relative = load_parameter[4]

            if load_parameter[1] == False:
                clientObject.distance_b_absolute = load_parameter[5]
            else:
                clientObject.distance_b_relative = load_parameter[5]

        elif load_distribution.name == "LOAD_DISTRIBUTION_PARABOLIC":
            clientObject.magnitude_1 = load_parameter[0]
            clientObject.magnitude_2 = load_parameter[1]
            clientObject.magnitude_3 = load_parameter[2]

        elif load_distribution.name == "LOAD_DISTRIBUTION_VARYING":
            clientObject.varying_load_parameters = model.clientModel.factory.create('ns0:member_load.varying_load_parameters')
            for i,j in enumerate(load_parameter):
                if len(load_parameter[i]) != 2:
                    raise ValueError("WARNING: MemberLoad no: %x, load case: %x - Wrong data input." % (no, load_case_no))
                mlvlp = model.clientModel.factory.create('ns0:member_load_varying_load_parameters_row')
                mlvlp.no = i+1
                mlvlp.row = model.clientModel.factory.create('ns0:member_load_varying_load_parameters')
                clearAttributes(mlvlp.row)
                mlvlp.row.distance = load_parameter[i][0]
                mlvlp.row.magnitude = load_parameter[i][1]

                clientObject.varying_load_parameters.member_load_varying_load_parameters.append(mlvlp)

        # Member Load Direction
        clientObject.load_direction = load_direction.name

        # Reference to List of Members
        clientObject.reference_to_list_of_members = list_reference

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Load Member Load to client model
        model.clientModel.service.set_member_load(load_case_no, clientObject)

    @staticmethod
    def Mass(
                no: int = 1,
                load_case_no: int = 1,
                members_no: str = '1',
                individual_mass_components: bool=False,
                mass_components: list = None,
                comment: str = '',
                params: dict = None,
                model = Model):
        """
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            members_no (str):Assigned Member(s)
            individual_mass_components (bool): Enable/Disable Individual Mass Components Option
            mass_components (list): Mass Components List
                if individual_mass_components == False:
                    mass_components = [M]
                else:
                    mass_components = [Mx, My, Mz, Ix, Iy, Iz]
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """
        # Client model | Member Load
        clientObject = model.clientModel.factory.create('ns0:member_load')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Load No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Members No. (e.g. '5 6 7 12')
        clientObject.members = ConvertToDlString(members_no)

        # Member Load Type
        clientObject.load_type = MemberLoadType.E_TYPE_MASS.name

        # Member Load Distribution
        clientObject.load_distribution= MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM.name

        # Individual Mass Components
        clientObject.individual_mass_components = individual_mass_components

        # Mass magnitude
        if individual_mass_components == False:
            clientObject.mass_global = mass_components[0]
        else:
            clientObject.mass_x = mass_components[0]
            clientObject.mass_y = mass_components[1]
            clientObject.mass_z = mass_components[2]

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Load Member Load to client model
        model.clientModel.service.set_member_load(load_case_no, clientObject)

    @staticmethod
    def Temperature(
                 no: int = 1,
                 load_case_no: int = 1,
                 members_no: str = '1',
                 load_distribution = MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM,
                 load_direction = MemberLoadDirection.LOAD_DIRECTION_LOCAL_Z,
                 load_parameter: list = None,
                 list_reference: bool= False,
                 load_over_total_length: bool= False,
                 comment: str = '',
                 params: dict = None,
                 model = Model):

        '''
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            members_no (str): Assigned Member(s)
            load_distribution (enum): Member Load Distribution Enumeration
            load_direction (enum): Member Load Direction Enumeration
            load_parameter (list/list of lists): Load Parameter List
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM:
                    load_parameter = [tt, tb]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TRAPEZIODAL:
                    for load_over_total_length == False:
                        load_parameter = [tt1, tt2, tb1, tb2, distance_a_relative = False, distance_a_relative = False, a_distance, b_distance]
                    for load_over_total_length == True:
                        load_parameter = [tt1, tt2, tb1, tb2]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TAPERED:
                    load_parameter = [tt1, tt2, tb1, tb2, distance_a_relative = False, distance_a_relative = False, a_distance, b_distance]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_PARABOLIC:
                    load_parameter = [tb1, tb2, tb3, tt1, tt2, tt3]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_VARYING:
                    load_parameter = [[distance, magnitude], ...]
            list_reference (bool): List Reference Boolean
            load_over_total_length (bool): Enable/Disable Load Over Total Length Option
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Member Load
        clientObject = model.clientModel.factory.create('ns0:member_load')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Load No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Members No. (e.g. '5 6 7 12')
        clientObject.members = ConvertToDlString(members_no)

        # Member Load Type
        load_type = MemberLoadType.LOAD_TYPE_TEMPERATURE
        clientObject.load_type = load_type.name

        # Member Load Distribution
        clientObject.load_distribution = load_distribution.name

        # Member Load Direction
        clientObject.load_direction = load_direction.name

        # Load Magnitude
        if load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM:
            clientObject.magnitude_t_b = load_parameter[0]
            clientObject.magnitude_t_t = load_parameter[1]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TRAPEZOIDAL:
            clientObject.magnitude_t_b_1 = load_parameter[0]
            clientObject.magnitude_t_b_2 = load_parameter[1]
            clientObject.magnitude_t_t_1 = load_parameter[2]
            clientObject.magnitude_t_t_2 = load_parameter[3]

            if load_over_total_length == False:

                if load_parameter[4] == True:
                    clientObject.distance_a_is_defined_as_relative = True
                    clientObject.distance_a_relative = load_parameter[6]
                else:
                    clientObject.distance_a_is_defined_as_relative = False
                    clientObject.distance_a_absolute = load_parameter[6]

                if load_parameter[5] == True:
                    clientObject.distance_b_is_defined_as_relative = True
                    clientObject.distance_b_relative = load_parameter[7]
                else:
                    clientObject.distance_b_is_defined_as_relative = False
                    clientObject.distance_b_absolute = load_parameter[7]
            else:
                clientObject.load_is_over_total_length = True

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TAPERED:

            clientObject.magnitude_t_b_1 = load_parameter[0]
            clientObject.magnitude_t_b_2 = load_parameter[1]
            clientObject.magnitude_t_t_1 = load_parameter[2]
            clientObject.magnitude_t_t_2 = load_parameter[3]

            if load_parameter[4] == True:
                clientObject.distance_a_is_defined_as_relative = True
                clientObject.distance_a_relative = load_parameter[6]
            else:
                clientObject.distance_a_is_defined_as_relative = False
                clientObject.distance_a_absolute = load_parameter[6]

            if load_parameter[5] == True:
                clientObject.distance_b_is_defined_as_relative = True
                clientObject.distance_b_relative = load_parameter[7]
            else:
                clientObject.distance_b_is_defined_as_relative = False
                clientObject.distance_b_absolute = load_parameter[7]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_PARABOLIC:

            clientObject.magnitude_t_b_1 = load_parameter[0]
            clientObject.magnitude_t_b_2 = load_parameter[1]
            clientObject.magnitude_t_b_3 = load_parameter[2]
            clientObject.magnitude_t_t_1 = load_parameter[3]
            clientObject.magnitude_t_t_2 = load_parameter[4]
            clientObject.magnitude_t_t_3 = load_parameter[5]

        elif load_distribution.name == "LOAD_DISTRIBUTION_VARYING":
            clientObject.varying_load_parameters = model.clientModel.factory.create('ns0:member_load.varying_load_parameters')
            for i,j in enumerate(load_parameter):
                if len(load_parameter[i]) != 3:
                    raise ValueError("WARNING: MemberLoad no: %x, load case: %x - Wrong data input." % (no, load_case_no))
                mlvlp = model.clientModel.factory.create('ns0:member_load_varying_load_parameters_row')
                mlvlp.no = i+1
                mlvlp.row = model.clientModel.factory.create('ns0:member_load_varying_load_parameters')
                clearAttributes(mlvlp.row)
                mlvlp.row.distance = load_parameter[i][0]
                mlvlp.row.magnitude = load_parameter[i][1]
                mlvlp.row.magnitude_t_c = load_parameter[i][1]
                mlvlp.row.magnitude_delta_t = load_parameter[i][2]
                mlvlp.row.magnitude_t_t = load_parameter[i][1]
                mlvlp.row.magnitude_t_b = load_parameter[i][2]

                clientObject.varying_load_parameters.member_load_varying_load_parameters.append(mlvlp)

        # Reference to list of members
        clientObject.reference_to_list_of_members = list_reference

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Load Member Load to client model
        model.clientModel.service.set_member_load(load_case_no, clientObject)

    @staticmethod
    def TemperatureChange(
                           no: int = 1,
                           load_case_no: int = 1,
                           members_no: str = '1',
                           load_distribution = MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM,
                           load_direction = MemberLoadDirection.LOAD_DIRECTION_LOCAL_Z,
                           load_parameter: list = None,
                           list_reference: bool= False,
                           load_over_total_length: bool= False,
                           comment: str = '',
                           params: dict = None,
                           model = Model):

        '''
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            members_no (str): Assigned Member(s)
            load_distribution (enum): Member Load Distribution
            load_direction (enum): Member Load Direction Enumeration
            load_parameter (list/list of lists): Load Parameter List
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM:
                    load_parameter = [tc, delta_t]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TRAPEZIODAL:
                    for load_over_total_length == False:
                        load_parameter = [delta_t_1, delta_t_2, t_c_1, t_c_2, distance_a_relative = False, distance_a_relative = False, a_distance, b_distance]
                    for load_over_total_length == True:
                        load_parameter = [delta_t_1, delta_t_2, t_c_1, t_c_2]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TAPERED:
                    load_parameter = [delta_t_1, delta_t_2, t_c_1, t_c_2, distance_a_relative = False, distance_a_relative = False, a_distance, b_distance]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_PARABOLIC:
                    load_parameter = [delta_t_1, delta_t_2, delta_t_3, t_c_1, t_c_2, t_c_3]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_VARYING:
                    load_parameter = [[distance, magnitude], ...]
            list_reference (bool): Enable/Disable List Reference Option
            load_over_total_length (bool): Enable/Disable Load Over Total Length Option
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Member Load
        clientObject = model.clientModel.factory.create('ns0:member_load')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Load No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Members No. (e.g. '5 6 7 12')
        clientObject.members = ConvertToDlString(members_no)

        # Member Load Type
        load_type = MemberLoadType.LOAD_TYPE_TEMPERATURE_CHANGE
        clientObject.load_type = load_type.name

        # Member Load Distribution
        clientObject.load_distribution = load_distribution.name

        # Member Load Direction
        clientObject.load_direction = load_direction.name

        # Load Magnitude
        if load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM:
            clientObject.magnitude_delta_t = load_parameter[0]
            clientObject.magnitude_t_c = load_parameter[1]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TRAPEZOIDAL:
            clientObject.magnitude_delta_t_1 = load_parameter[0]
            clientObject.magnitude_delta_t_2 = load_parameter[1]
            clientObject.magnitude_t_c_1 = load_parameter[2]
            clientObject.magnitude_t_c_2 = load_parameter[3]

            if load_over_total_length == False:

                if load_parameter[4] == True:
                    clientObject.distance_a_is_defined_as_relative = True
                    clientObject.distance_a_relative = load_parameter[6]
                else:
                    clientObject.distance_a_is_defined_as_relative = False
                    clientObject.distance_a_absolute = load_parameter[6]

                if load_parameter[5] == True:
                    clientObject.distance_b_is_defined_as_relative = True
                    clientObject.distance_b_relative = load_parameter[7]
                else:
                    clientObject.distance_b_is_defined_as_relative = False
                    clientObject.distance_b_absolute = load_parameter[7]
            else:
                clientObject.load_is_over_total_length = True

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TAPERED:

            clientObject.magnitude_delta_t_1 = load_parameter[0]
            clientObject.magnitude_delta_t_2 = load_parameter[1]
            clientObject.magnitude_t_c_1 = load_parameter[2]
            clientObject.magnitude_t_c_2 = load_parameter[3]

            if load_parameter[4] == True:
                clientObject.distance_a_is_defined_as_relative = True
                clientObject.distance_a_relative = load_parameter[6]
            else:
                clientObject.distance_a_is_defined_as_relative = False
                clientObject.distance_a_absolute = load_parameter[6]

            if load_parameter[5] == True:
                clientObject.distance_b_is_defined_as_relative = True
                clientObject.distance_b_relative = load_parameter[7]
            else:
                clientObject.distance_b_is_defined_as_relative = False
                clientObject.distance_b_absolute = load_parameter[7]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_PARABOLIC:

            clientObject.magnitude_delta_t_1 = load_parameter[0]
            clientObject.magnitude_delta_t_2 = load_parameter[1]
            clientObject.magnitude_delta_t_3 = load_parameter[2]
            clientObject.magnitude_t_c_1 = load_parameter[3]
            clientObject.magnitude_t_c_2 = load_parameter[4]
            clientObject.magnitude_t_c_3 = load_parameter[5]

        elif load_distribution.name == "LOAD_DISTRIBUTION_VARYING":
            clientObject.varying_load_parameters = model.clientModel.factory.create('ns0:member_load.varying_load_parameters')
            for i,j in enumerate(load_parameter):
                if len(load_parameter[i]) != 3:
                    raise ValueError("WARNING: MemberLoad no: %x, load case: %x - Wrong data input." % (no, load_case_no))
                mlvlp = model.clientModel.factory.create('ns0:member_load_varying_load_parameters_row')
                mlvlp.no = i+1
                mlvlp.row = model.clientModel.factory.create('ns0:member_load_varying_load_parameters')
                clearAttributes(mlvlp.row)
                mlvlp.row.distance = load_parameter[i][0]
                mlvlp.row.magnitude = load_parameter[i][1]
                mlvlp.row.magnitude_t_c = load_parameter[i][1]
                mlvlp.row.magnitude_delta_t = load_parameter[i][2]
                mlvlp.row.magnitude_t_t = load_parameter[i][1]
                mlvlp.row.magnitude_t_b = load_parameter[i][2]

                clientObject.varying_load_parameters.member_load_varying_load_parameters.append(mlvlp)

        # Reference to list of members
        clientObject.reference_to_list_of_members = list_reference

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Load Member Load to client model
        model.clientModel.service.set_member_load(load_case_no, clientObject)

    @staticmethod
    def AxialStrain(
                    no: int = 1,
                    load_case_no: int = 1,
                    members_no: str = '1',
                    load_distribution = MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM,
                    load_direction = MemberLoadDirection.LOAD_DIRECTION_LOCAL_X,
                    load_parameter: list = None,
                    list_reference: bool= False,
                    load_over_total_length: bool= False,
                    comment: str = '',
                    params: dict = None,
                    model = Model):

        '''
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            members_no (str): Assigned Member(s)
            load_distribution (enum): Member Load Distribution Enumeration
            load_direction (enum): Member Load Direction Enumeration
            load_parameter (list/list of lists): Load Parameter List
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM:
                    load_parameter = [epsilon]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TRAPEZIODAL:
                    load_parameter = [epsilon1, epsilon2, distance_a_relative = False, distance_a_relative = False, a_distance, b_distance]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TAPERED:
                    load_parameter = [epsilon1, epsilon2, distance_a_relative = False, distance_a_relative = False, a_distance, b_distance]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TAPERED:
                    load_parameter = [epsilon1, epsilon2, epsilon3]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_VARYING:
                    load_parameter = [[distance, magnitude], ...]
            list_reference (bool): Enable/Disable List Reference Option
            load_over_total_length (bool): Enable/Disable Load Over Total Length Option
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Member Load
        clientObject = model.clientModel.factory.create('ns0:member_load')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Load No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Members No. (e.g. '5 6 7 12')
        clientObject.members = ConvertToDlString(members_no)

        # Member Load Type
        load_type = MemberLoadType.LOAD_TYPE_AXIAL_STRAIN
        clientObject.load_type = load_type.name

        # Member Load Distribution
        clientObject.load_distribution = load_distribution.name

        # Member Load Direction
        clientObject.load_direction = load_direction.name

        # Load Magnitude
        if load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM:
            clientObject.magnitude = load_parameter[0]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TRAPEZOIDAL:
            clientObject.magnitude_1 = load_parameter[0]
            clientObject.magnitude_2 = load_parameter[1]

            if load_over_total_length == False:

                if load_parameter[2] == True:
                    clientObject.distance_a_is_defined_as_relative = True
                    clientObject.distance_a_relative = load_parameter[4]
                else:
                    clientObject.distance_a_is_defined_as_relative = False
                    clientObject.distance_a_absolute = load_parameter[4]

                if load_parameter[3] == True:
                    clientObject.distance_b_is_defined_as_relative = True
                    clientObject.distance_b_relative = load_parameter[5]
                else:
                    clientObject.distance_b_is_defined_as_relative = False
                    clientObject.distance_b_absolute = load_parameter[5]
            else:
                clientObject.load_is_over_total_length = True

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TAPERED:

            clientObject.magnitude_1 = load_parameter[0]
            clientObject.magnitude_2 = load_parameter[1]

            if load_parameter[2] == True:
                clientObject.distance_a_is_defined_as_relative = True
                clientObject.distance_a_relative = load_parameter[4]
            else:
                clientObject.distance_a_is_defined_as_relative = False
                clientObject.distance_a_absolute = load_parameter[4]

            if load_parameter[3] == True:
                clientObject.distance_b_is_defined_as_relative = True
                clientObject.distance_b_relative = load_parameter[5]
            else:
                clientObject.distance_b_is_defined_as_relative = False
                clientObject.distance_b_absolute = load_parameter[5]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_PARABOLIC:

            clientObject.magnitude_1 = load_parameter[0]
            clientObject.magnitude_2 = load_parameter[1]
            clientObject.magnitude_3 = load_parameter[2]

        elif load_distribution.name == "LOAD_DISTRIBUTION_VARYING":
            clientObject.varying_load_parameters = model.clientModel.factory.create('ns0:member_load.varying_load_parameters')
            for i,j in enumerate(load_parameter):
                if len(load_parameter[i]) != 3:
                    raise ValueError("WARNING: MemberLoad no: %x, load case: %x - Wrong data input." % (no, load_case_no))
                mlvlp = model.clientModel.factory.create('ns0:member_load_varying_load_parameters_row')
                mlvlp.no = i+1
                mlvlp.row = model.clientModel.factory.create('ns0:member_load_varying_load_parameters')
                clearAttributes(mlvlp.row)
                mlvlp.row.distance = load_parameter[i][0]
                mlvlp.row.magnitude = load_parameter[i][1]

                clientObject.varying_load_parameters.member_load_varying_load_parameters.append(mlvlp)

        # Reference to list of members
        clientObject.reference_to_list_of_members = list_reference

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Load Member Load to client model
        model.clientModel.service.set_member_load(load_case_no, clientObject)

    @staticmethod
    def AxialDisplacement(
                    no: int = 1,
                    load_case_no: int = 1,
                    members_no: str = '1',
                    load_direction = MemberLoadDirection.LOAD_DIRECTION_LOCAL_X,
                    magnitude : float = 0.0,
                    comment: str = '',
                    params: dict = None,
                    model = Model):

        '''
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            members_no (str): Assigned Member(s)
            load_direction (enum): Member Load Direction Enumeration
            magnitude (float): Load Magnitude
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Member Load
        clientObject = model.clientModel.factory.create('ns0:member_load')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Load No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Members No. (e.g. '5 6 7 12')
        clientObject.members = ConvertToDlString(members_no)

        # Member Load Type
        load_type = MemberLoadType.LOAD_TYPE_AXIAL_DISPLACEMENT
        clientObject.load_type = load_type.name

        # Member Load Distribution
        clientObject.load_distribution = MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM.name

        # Member Load Direction
        clientObject.load_direction = load_direction.name

        # Load Magnitude
        clientObject.magnitude = magnitude

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Load Member Load to client model
        model.clientModel.service.set_member_load(load_case_no, clientObject)

    @staticmethod
    def Precamber(
                 no: int = 1,
                 load_case_no: int = 1,
                 members_no: str = '1',
                 load_distribution = MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM,
                 load_direction = MemberLoadDirection.LOAD_DIRECTION_LOCAL_Z,
                 load_parameter: list = None,
                 list_reference: bool= False,
                 load_over_total_length: bool= False,
                 comment: str = '',
                 params: dict = None,
                 model = Model):

        '''
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            members_no (str): Assigned Member(s)
            load_distribution (enum): Member Load Distribution Enumeration
            load_direction (enum): Member Load Direction Enumeration
            load_parameter (list/list of lists): Load Parameter List
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM:
                    load_parameter = [magnitude]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TRAPEZIODAL:
                    load_parameter = [magnitude_1, magnitude_2, distance_a_relative = False, distance_a_relative = False, a_distance, b_distance]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TAPERED:
                    load_parameter = [magnitude_1, magnitude_2, distance_a_relative = False, distance_a_relative = False, a_distance, b_distance]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_PARABOLIC:
                    load_parameter = [magnitude_1, magnitude_2, magnitude_3]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_VARYING:
                    load_parameter = [[distance, magnitude], ...]
            list_reference (bool): Enable/Disable List Reference Option
            load_over_total_length (bool): Enable/Disable Load Over Total Length Option
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Member Load
        clientObject = model.clientModel.factory.create('ns0:member_load')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Load No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Members No. (e.g. '5 6 7 12')
        clientObject.members = ConvertToDlString(members_no)

        # Member Load Type
        load_type = MemberLoadType.LOAD_TYPE_PRECAMBER
        clientObject.load_type = load_type.name

        # Member Load Distribution
        clientObject.load_distribution = load_distribution.name

        # Member Load Direction
        clientObject.load_direction = load_direction.name

        # Load Magnitude
        if load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM:
            clientObject.magnitude = load_parameter[0]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TRAPEZOIDAL:
            clientObject.magnitude_1 = load_parameter[0]
            clientObject.magnitude_2 = load_parameter[1]

            if load_over_total_length == False:

                if load_parameter[2] == True:
                    clientObject.distance_a_is_defined_as_relative = True
                    clientObject.distance_a_relative = load_parameter[4]
                else:
                    clientObject.distance_a_is_defined_as_relative = False
                    clientObject.distance_a_absolute = load_parameter[4]

                if load_parameter[3] == True:
                    clientObject.distance_b_is_defined_as_relative = True
                    clientObject.distance_b_relative = load_parameter[5]
                else:
                    clientObject.distance_b_is_defined_as_relative = False
                    clientObject.distance_b_absolute = load_parameter[5]
            else:
                clientObject.load_is_over_total_length = True

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TAPERED:

            clientObject.magnitude_1 = load_parameter[0]
            clientObject.magnitude_2 = load_parameter[1]

            if load_parameter[2] == True:
                clientObject.distance_a_is_defined_as_relative = True
                clientObject.distance_a_relative = load_parameter[4]
            else:
                clientObject.distance_a_is_defined_as_relative = False
                clientObject.distance_a_absolute = load_parameter[4]

            if load_parameter[3] == True:
                clientObject.distance_b_is_defined_as_relative = True
                clientObject.distance_b_relative = load_parameter[5]
            else:
                clientObject.distance_b_is_defined_as_relative = False
                clientObject.distance_b_absolute = load_parameter[5]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_PARABOLIC:

            clientObject.magnitude_1 = load_parameter[0]
            clientObject.magnitude_2 = load_parameter[1]
            clientObject.magnitude_3 = load_parameter[2]

        elif load_distribution.name == "LOAD_DISTRIBUTION_VARYING":
            clientObject.varying_load_parameters = model.clientModel.factory.create('ns0:member_load.varying_load_parameters')
            for i,j in enumerate(load_parameter):
                if len(load_parameter[i]) != 2:
                    raise ValueError("WARNING: MemberLoad no: %x, load case: %x - Wrong data input." % (no, load_case_no))
                mlvlp = model.clientModel.factory.create('ns0:member_load_varying_load_parameters_row')
                mlvlp.no = i+1
                mlvlp.row = model.clientModel.factory.create('ns0:member_load_varying_load_parameters')
                clearAttributes(mlvlp.row)
                mlvlp.row.distance = load_parameter[i][0]
                mlvlp.row.magnitude = load_parameter[i][1]

                clientObject.varying_load_parameters.member_load_varying_load_parameters.append(mlvlp)

        # Reference to list of members
        clientObject.reference_to_list_of_members = list_reference

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Load Member Load to client model
        model.clientModel.service.set_member_load(load_case_no, clientObject)

    @staticmethod
    def InitialPrestress(
                 no: int = 1,
                 load_case_no: int = 1,
                 members_no: str = '1',
                 load_direction = MemberLoadDirection.LOAD_DIRECTION_LOCAL_X,
                 magnitude : float = 0.0,
                 comment: str = '',
                 params: dict = None,
                 model = Model):

        '''
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            members_no (str): Assigned Member(s)
            load_direction (enum): Member Load Direction Enumeration
            magnitude (float): Load Magnitude
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Member Load
        clientObject = model.clientModel.factory.create('ns0:member_load')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Load No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Members No. (e.g. '5 6 7 12')
        clientObject.members = ConvertToDlString(members_no)

        # Member Load Type
        load_type = MemberLoadType.LOAD_TYPE_INITIAL_PRESTRESS
        clientObject.load_type = load_type.name

        # Member Load Distribution
        clientObject.load_distribution = MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM.name

        # Member Load Direction
        clientObject.load_direction = load_direction.name

        # Load Magnitude
        clientObject.magnitude = magnitude

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Load Member Load to client model
        model.clientModel.service.set_member_load(load_case_no, clientObject)

    @staticmethod
    def Displacement(
                 no: int = 1,
                 load_case_no: int = 1,
                 members_no: str = '1',
                 load_distribution = MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM,
                 load_direction = MemberLoadDirection.LOAD_DIRECTION_LOCAL_Z,
                 load_parameter: list = None,
                 list_reference: bool= False,
                 load_over_total_length: bool= False,
                 comment: str = '',
                 params: dict = None,
                 model = Model):

        '''
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            members_no (str): Assigned Member(s)
            load_distribution (enum): Member Load Distribution Enumeration
            load_direction (enum): Member Load Direction Enumeration
            load_parameter (list/list of lists): Load Parameter List
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM:
                    load_parameter = [magnitude]
                for load_distrubition == MemberLoadDistribution.LOAD_DISTRIBUTION_CONCENTRATED_1:
                    load_parameter = [magnitude, distance_a_is_defined_as_relative = False, distance_a]
                for load_distrubition == MemberLoadDistribution.LOAD_DISTRIBUTION_CONCENTRATED_N:
                    load_parameter = [magnitude, distance_a_is_defined_as_relative = False, distance_b_is_defined_as_relative = False, distance_a, distance_b]
                for load_distrubition == MemberLoadDistribution.LOAD_DISTRIBUTION_CONCENTRATED_2x2:
                    load_parameter = [magnitude, distance_a_is_defined_as_relative = False, distance_b_is_defined_as_relative = False, distance_c_is_defined_as_relative = False, distance_a, distance_b, distance_c]
                for load_distrubition == MemberLoadDistribution.LOAD_DISTRIBUTION_CONCENTRATED_2:
                    load_parameter = [magnitude_1, magnitude_2, distance_a_is_defined_as_relative = False, distance_b_is_defined_as_relative = False, distance_a, distance_b]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_CONCENTRATED_VARYING:
                    load_parameter = [[distance, magnitude], ...]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TRAPEZIODAL:
                    load_parameter = [magnitude_1, magnitude_2, distance_a_relative = False, distance_a_relative = False, a_distance, b_distance]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TAPERED:
                    load_parameter = [magnitude_1, magnitude_2, distance_a_relative = False, distance_a_relative = False, a_distance, b_distance]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_PARABOLIC:
                    load_parameter = [magnitude_1, magnitude_2, magnitude_3]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_VARYING:
                    load_parameter = [[distance, magnitude], ...]
            list_reference (bool): Enable/Disable List Reference Option
            load_over_total_length (bool): Enable/Disable Load Over Total Length Option
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Member Load
        clientObject = model.clientModel.factory.create('ns0:member_load')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Load No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Members No. (e.g. '5 6 7 12')
        clientObject.members = ConvertToDlString(members_no)

        # Member Load Type
        load_type = MemberLoadType.LOAD_TYPE_DISPLACEMENT
        clientObject.load_type = load_type.name

        # Member Load Distribution
        clientObject.load_distribution = load_distribution.name

        # Member Load Direction
        clientObject.load_direction = load_direction.name

        # Load Magnitude
        if load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM:
            clientObject.magnitude = load_parameter[0]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_CONCENTRATED_1:
            clientObject.magnitude = load_parameter[0]
            clientObject.distance_a_is_defined_as_relative = load_parameter[1]
            if load_parameter[1]:
                clientObject.distance_a_relative = load_parameter[2]
            else:
                clientObject.distance_a_absolute = load_parameter[2]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_CONCENTRATED_N:
            clientObject.magnitude = load_parameter[0]
            clientObject.distance_a_is_defined_as_relative = load_parameter[1]
            clientObject.distance_b_is_defined_as_relative = load_parameter[2]

            if load_parameter[1]:
                clientObject.distance_a_relative = load_parameter[3]
            else:
                clientObject.distance_a_absolute = load_parameter[3]

            if load_parameter[2]:
                clientObject.distance_b_relative = load_parameter[4]
            else:
                clientObject.distance_b_absolute = load_parameter[4]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_CONCENTRATED_2x2:
            clientObject.magnitude = load_parameter[0]
            clientObject.distance_a_is_defined_as_relative = load_parameter[1]
            clientObject.distance_b_is_defined_as_relative = load_parameter[2]
            clientObject.distance_c_is_defined_as_relative = load_parameter[3]

            if load_parameter[1]:
                clientObject.distance_a_relative = load_parameter[4]
            else:
                clientObject.distance_a_absolute = load_parameter[4]

            if load_parameter[2]:
                clientObject.distance_b_relative = load_parameter[5]
            else:
                clientObject.distance_b_absolute = load_parameter[5]

            if load_parameter[3]:
                clientObject.distance_c_relative = load_parameter[6]
            else:
                clientObject.distance_c_absolute = load_parameter[6]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_CONCENTRATED_2:
            clientObject.magnitude_1 = load_parameter[0]
            clientObject.magnitude_2 = load_parameter[1]

            clientObject.distance_a_is_defined_as_relative = load_parameter[2]
            clientObject.distance_b_is_defined_as_relative = load_parameter[3]

            if load_parameter[2]:
                clientObject.distance_a_relative = load_parameter[4]
            else:
                clientObject.distance_a_absolute = load_parameter[4]

            if load_parameter[3]:
                clientObject.distance_b_relative = load_parameter[5]
            else:
                clientObject.distance_b_absolute = load_parameter[5]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_CONCENTRATED_VARYING:
            clientObject.varying_load_parameters = model.clientModel.factory.create('ns0:member_load.varying_load_parameters')
            for i,j in enumerate(load_parameter):
                if len(load_parameter[i]) != 3:
                    raise ValueError("WARNING: MemberLoad no: %x, load case: %x - Wrong data input." % (no, load_case_no))
                mlvlp = model.clientModel.factory.create('ns0:member_load_varying_load_parameters_row')
                mlvlp.no = i+1
                mlvlp.row = model.clientModel.factory.create('ns0:member_load_varying_load_parameters')
                clearAttributes(mlvlp.row)
                mlvlp.row.distance = load_parameter[i][0]
                mlvlp.row.magnitude = load_parameter[i][1]

                clientObject.varying_load_parameters.member_load_varying_load_parameters.append(mlvlp)

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TRAPEZOIDAL:
            clientObject.magnitude_1 = load_parameter[0]
            clientObject.magnitude_2 = load_parameter[1]

            if load_over_total_length == False:

                clientObject.distance_a_is_defined_as_relative = load_parameter[2]
                clientObject.distance_b_is_defined_as_relative = load_parameter[3]

                if load_parameter[2]:
                    clientObject.distance_a_relative = load_parameter[4]
                else:
                    clientObject.distance_a_absolute = load_parameter[4]

                if load_parameter[3]:
                    clientObject.distance_b_relative = load_parameter[5]
                else:
                    clientObject.distance_b_absolute = load_parameter[5]

            else:
                clientObject.load_is_over_total_length = True

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TAPERED:
            clientObject.magnitude_1 = load_parameter[0]
            clientObject.magnitude_2 = load_parameter[1]

            clientObject.distance_a_is_defined_as_relative = load_parameter[2]
            clientObject.distance_b_is_defined_as_relative = load_parameter[3]

            if load_parameter[2]:
                clientObject.distance_a_relative = load_parameter[4]
            else:
                clientObject.distance_a_absolute = load_parameter[4]

            if load_parameter[3]:
                clientObject.distance_b_relative = load_parameter[5]
            else:
                clientObject.distance_b_absolute = load_parameter[5]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_PARABOLIC:
            clientObject.magnitude_1 = load_parameter[0]
            clientObject.magnitude_2 = load_parameter[1]
            clientObject.magnitude_3 = load_parameter[2]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_VARYING:
            clientObject.varying_load_parameters = model.clientModel.factory.create('ns0:member_load.varying_load_parameters')
            for i,j in enumerate(load_parameter):
                if len(load_parameter[i]) != 2:
                    raise ValueError("WARNING: MemberLoad no: %x, load case: %x - Wrong data input." % (no, load_case_no))
                mlvlp = model.clientModel.factory.create('ns0:member_load_varying_load_parameters_row')
                mlvlp.no = i+1
                mlvlp.row = model.clientModel.factory.create('ns0:member_load_varying_load_parameters')
                clearAttributes(mlvlp.row)
                mlvlp.row.distance = load_parameter[i][0]
                mlvlp.row.magnitude = load_parameter[i][1]

                clientObject.varying_load_parameters.member_load_varying_load_parameters.append(mlvlp)

        # Reference to list of members
        clientObject.reference_to_list_of_members = list_reference

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Load Member Load to client model
        model.clientModel.service.set_member_load(load_case_no, clientObject)

    @staticmethod
    def Rotation(
                 no: int = 1,
                 load_case_no: int = 1,
                 members_no: str = '1',
                 load_distribution = MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM,
                 load_direction = MemberLoadDirection.LOAD_DIRECTION_LOCAL_Z,
                 load_parameter: list = None,
                 list_reference: bool= False,
                 load_over_total_length: bool= False,
                 comment: str = '',
                 params: dict = None,
                 model = Model):

        '''
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            members_no (str): Assigned Member(s)
            load_distribution (enum): Member Load Distribution Enumeration
            load_direction (enum): Member Load Direction Enumeration
            load_parameter (list/list of lists): Load Parameter List
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM:
                    load_parameter = [magnitude]
                for load_distrubition == MemberLoadDistribution.LOAD_DISTRIBUTION_CONCENTRATED_1:
                    load_parameter = [magnitude, distance_a_is_defined_as_relative = False, distance_a]
                for load_distrubition == MemberLoadDistribution.LOAD_DISTRIBUTION_CONCENTRATED_N:
                    load_parameter = [magnitude, distance_a_is_defined_as_relative = False, distance_b_is_defined_as_relative = False, distance_a, distance_b]
                for load_distrubition == MemberLoadDistribution.LOAD_DISTRIBUTION_CONCENTRATED_2x2:
                    load_parameter = [magnitude, distance_a_is_defined_as_relative = False, distance_b_is_defined_as_relative = False, distance_c_is_defined_as_relative = False, distance_a, distance_b, distance_c]
                for load_distrubition == MemberLoadDistribution.LOAD_DISTRIBUTION_CONCENTRATED_2:
                    load_parameter = [magnitude_1, magnitude_2, distance_a_is_defined_as_relative = False, distance_b_is_defined_as_relative = False, distance_a, distance_b]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_CONCENTRATED_VARYING:
                    load_parameter = [[distance, magnitude], ...]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TRAPEZIODAL:
                    load_parameter = [magnitude_1, magnitude_2, distance_a_relative = False, distance_a_relative = False, a_distance, b_distance]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TAPERED:
                    load_parameter = [magnitude_1, magnitude_2, distance_a_relative = False, distance_a_relative = False, a_distance, b_distance]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_PARABOLIC:
                    load_parameter = [magnitude_1, magnitude_2, magnitude_3]
                for load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_VARYING:
                    load_parameter = [[distance, magnitude], ...]
            list_reference (bool): Enable/Disable List Reference Option
            load_over_total_length (bool): Enable/Disable Load Over Total Length Option
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Member Load
        clientObject = model.clientModel.factory.create('ns0:member_load')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Load No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Members No. (e.g. '5 6 7 12')
        clientObject.members = ConvertToDlString(members_no)

        # Member Load Type
        load_type = MemberLoadType.LOAD_TYPE_ROTATION
        clientObject.load_type = load_type.name

        # Member Load Distribution
        clientObject.load_distribution = load_distribution.name

        # Member Load Direction
        clientObject.load_direction = load_direction.name

        # Load Magnitude
        if load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM:
            clientObject.magnitude = load_parameter[0]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_CONCENTRATED_1:
            clientObject.magnitude = load_parameter[0]
            clientObject.distance_a_is_defined_as_relative = load_parameter[1]
            if load_parameter[1]:
                clientObject.distance_a_relative = load_parameter[2]
            else:
                clientObject.distance_a_absolute = load_parameter[2]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_CONCENTRATED_N:
            clientObject.magnitude = load_parameter[0]
            clientObject.distance_a_is_defined_as_relative = load_parameter[1]
            clientObject.distance_b_is_defined_as_relative = load_parameter[2]

            if load_parameter[1]:
                clientObject.distance_a_relative = load_parameter[3]
            else:
                clientObject.distance_a_absolute = load_parameter[3]

            if load_parameter[2]:
                clientObject.distance_b_relative = load_parameter[4]
            else:
                clientObject.distance_b_absolute = load_parameter[4]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_CONCENTRATED_2x2:
            clientObject.magnitude = load_parameter[0]
            clientObject.distance_a_is_defined_as_relative = load_parameter[1]
            clientObject.distance_b_is_defined_as_relative = load_parameter[2]
            clientObject.distance_c_is_defined_as_relative = load_parameter[3]

            if load_parameter[1]:
                clientObject.distance_a_relative = load_parameter[4]
            else:
                clientObject.distance_a_absolute = load_parameter[4]

            if load_parameter[2]:
                clientObject.distance_b_relative = load_parameter[5]
            else:
                clientObject.distance_b_absolute = load_parameter[5]

            if load_parameter[3]:
                clientObject.distance_c_relative = load_parameter[6]
            else:
                clientObject.distance_c_absolute = load_parameter[6]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_CONCENTRATED_2:
            clientObject.magnitude_1 = load_parameter[0]
            clientObject.magnitude_2 = load_parameter[1]

            clientObject.distance_a_is_defined_as_relative = load_parameter[2]
            clientObject.distance_b_is_defined_as_relative = load_parameter[3]

            if load_parameter[2]:
                clientObject.distance_a_relative = load_parameter[4]
            else:
                clientObject.distance_a_absolute = load_parameter[4]

            if load_parameter[3]:
                clientObject.distance_b_relative = load_parameter[5]
            else:
                clientObject.distance_b_absolute = load_parameter[5]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_CONCENTRATED_VARYING:
            clientObject.varying_load_parameters = model.clientModel.factory.create('ns0:member_load.varying_load_parameters')
            for i,j in enumerate(load_parameter):
                if len(load_parameter[i]) != 2:
                    raise ValueError("WARNING: MemberLoad no: %x, load case: %x - Wrong data input." % (no, load_case_no))
                mlvlp = model.clientModel.factory.create('ns0:member_load_varying_load_parameters_row')
                mlvlp.no = i+1
                mlvlp.row = model.clientModel.factory.create('ns0:member_load_varying_load_parameters')
                clearAttributes(mlvlp.row)
                mlvlp.row.distance = load_parameter[i][0]
                mlvlp.row.magnitude = load_parameter[i][1]

                clientObject.varying_load_parameters.member_load_varying_load_parameters.append(mlvlp)

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TRAPEZOIDAL:
            clientObject.magnitude_1 = load_parameter[0]
            clientObject.magnitude_2 = load_parameter[1]

            if load_over_total_length == False:

                clientObject.distance_a_is_defined_as_relative = load_parameter[2]
                clientObject.distance_b_is_defined_as_relative = load_parameter[3]

                if load_parameter[2]:
                    clientObject.distance_a_relative = load_parameter[4]
                else:
                    clientObject.distance_a_absolute = load_parameter[4]

                if load_parameter[3]:
                    clientObject.distance_b_relative = load_parameter[5]
                else:
                    clientObject.distance_b_absolute = load_parameter[5]

            else:
                clientObject.load_is_over_total_length = True

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_TAPERED:
            clientObject.magnitude_1 = load_parameter[0]
            clientObject.magnitude_2 = load_parameter[1]

            clientObject.distance_a_is_defined_as_relative = load_parameter[2]
            clientObject.distance_b_is_defined_as_relative = load_parameter[3]

            if load_parameter[2]:
                clientObject.distance_a_relative = load_parameter[4]
            else:
                clientObject.distance_a_absolute = load_parameter[4]

            if load_parameter[3]:
                clientObject.distance_b_relative = load_parameter[5]
            else:
                clientObject.distance_b_absolute = load_parameter[5]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_PARABOLIC:
            clientObject.magnitude_1 = load_parameter[0]
            clientObject.magnitude_2 = load_parameter[1]
            clientObject.magnitude_3 = load_parameter[2]

        elif load_distribution == MemberLoadDistribution.LOAD_DISTRIBUTION_VARYING:
            clientObject.varying_load_parameters = model.clientModel.factory.create('ns0:member_load.varying_load_parameters')
            for i,j in enumerate(load_parameter):
                if len(load_parameter[i]) != 2:
                    raise ValueError("WARNING: MemberLoad no: %x, load case: %x - Wrong data input." % (no, load_case_no))
                mlvlp = model.clientModel.factory.create('ns0:member_load_varying_load_parameters_row')
                mlvlp.no = i+1
                mlvlp.row = model.clientModel.factory.create('ns0:member_load_varying_load_parameters')
                clearAttributes(mlvlp.row)
                mlvlp.row.distance = load_parameter[i][0]
                mlvlp.row.magnitude = load_parameter[i][1]

                clientObject.varying_load_parameters.member_load_varying_load_parameters.append(mlvlp)

        # Reference to list of members
        clientObject.reference_to_list_of_members = list_reference

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Load Member Load to client model
        model.clientModel.service.set_member_load(load_case_no, clientObject)

    @staticmethod
    def PipeContentFull(
                 no: int = 1,
                 load_case_no: int = 1,
                 members_no: str = '1',
                 load_direction_orientation = MemberLoadDirectionOrientation.LOAD_DIRECTION_FORWARD,
                 specific_weight : float = 0.0,
                 comment: str = '',
                 params: dict = None,
                 model = Model):

        '''
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            members_no (str): Assigned Member(s)
            load_direction_orientation (enum): Member Load Direction Enumeration
            specific_weight (float): Specific Weight
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Member Load
        clientObject = model.clientModel.factory.create('ns0:member_load')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Load No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Members No. (e.g. '5 6 7 12')
        clientObject.members = ConvertToDlString(members_no)

        # Member Load Type
        load_type = MemberLoadType.LOAD_TYPE_PIPE_CONTENT_FULL
        clientObject.load_type = load_type.name

        # Member Load Distribution
        clientObject.load_distribution = MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM.name

        # Member Load Direction
        clientObject.load_direction = MemberLoadDirection.LOAD_DIRECTION_GLOBAL_Z_OR_USER_DEFINED_W_TRUE.name

        # Member Load Orientation
        clientObject.load_direction_orientation = load_direction_orientation.name

        # Load Magnitude
        clientObject.magnitude = specific_weight

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Load Member Load to client model
        model.clientModel.service.set_member_load(load_case_no, clientObject)

    @staticmethod
    def PipeContentPartial(
                 no: int = 1,
                 load_case_no: int = 1,
                 members_no: str = '1',
                 load_direction_orientation = MemberLoadDirectionOrientation.LOAD_DIRECTION_FORWARD,
                 specific_weight : float = 0.0,
                 filling_height : float = 0.0,
                 comment: str = '',
                 params: dict = None,
                 model = Model):

        '''
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            members_no (str): Assigned Member(s)
            load_direction_orientation (enum): Member Load Direction Enumeration
            specific_weight (float): Specific Weight
            filling_height (float): Filling Height
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Member Load
        clientObject = model.clientModel.factory.create('ns0:member_load')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Load No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Members No. (e.g. '5 6 7 12')
        clientObject.members = ConvertToDlString(members_no)

        # Member Load Type
        load_type = MemberLoadType.LOAD_TYPE_PIPE_CONTENT_PARTIAL
        clientObject.load_type = load_type.name

        # Member Load Distribution
        clientObject.load_distribution = MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM.name

        # Member Load Direction
        clientObject.load_direction = MemberLoadDirection.LOAD_DIRECTION_GLOBAL_Z_OR_USER_DEFINED_W_TRUE.name

        # Member Load Orientation
        clientObject.load_direction_orientation = load_direction_orientation.name

        # Load Magnitude
        clientObject.magnitude = specific_weight

        # Filling Height
        clientObject.filling_height = filling_height

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Load Member Load to client model
        model.clientModel.service.set_member_load(load_case_no, clientObject)

    @staticmethod
    def PipeInternalPressure(
                 no: int = 1,
                 load_case_no: int = 1,
                 members_no: str = '1',
                 pressure : float = 0.0,
                 comment: str = '',
                 params: dict = None,
                 model = Model):

        '''
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            members_no (str): Assigned Member(s)
            pressure (float): Pressure
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Member Load
        clientObject = model.clientModel.factory.create('ns0:member_load')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Load No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Members No. (e.g. '5 6 7 12')
        clientObject.members = ConvertToDlString(members_no)

        # Member Load Type
        load_type = MemberLoadType.LOAD_TYPE_PIPE_INTERNAL_PRESSURE
        clientObject.load_type = load_type.name

        # Member Load Distribution
        clientObject.load_distribution = MemberLoadDistribution.LOAD_DISTRIBUTION_UNIFORM.name

        # Member Load Direction
        clientObject.load_direction = MemberLoadDirection.LOAD_DIRECTION_LOCAL_X.name

        # Load Magnitude
        clientObject.magnitude = pressure

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Load Member Load to client model
        model.clientModel.service.set_member_load(load_case_no, clientObject)

    @staticmethod
    def RotaryMotion(
                 no: int = 1,
                 load_case_no: int = 1,
                 members_no: str = '1',
                 angular_acceleration : float = 1.0,
                 angular_velocity : float = 2.0,
                 axis_definition_type = MemberLoadAxisDefinitionType.AXIS_DEFINITION_TWO_POINTS,
                 axis_orientation = MemberLoadAxisDefinitionAxisOrientation.AXIS_POSITIVE,
                 axis_definition = MemberLoadAxisDefinition.AXIS_X,
                 axis_definition_p1: list = [1,0,1],
                 axis_definition_p2: list = [0,1,0],
                 comment: str = '',
                 params: dict = None,
                 model = Model):

        '''
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            members_no (str): Assigned Member(s)
            angular_acceleration (float): Angular Acceleration
            angular_velocity (float): Angular Velocity
            axis_definition_type (enum): Member Load Axis Definition Type Enumeration
            axis_orientation (enum): Member Load Axis Orientation Enumeration
            axis_definition (enum): Member Load Axis Definition Enumeration
            axis_definition_p1 (list): P1 List [X, Y, Z]
            axis_definition_p2 (list): P2 List [X, Y, Z]
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Member Load
        clientObject = model.clientModel.factory.create('ns0:member_load')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Load No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Members No. (e.g. '5 6 7 12')
        clientObject.members = ConvertToDlString(members_no)

        # Member Load Type
        load_type = MemberLoadType.LOAD_TYPE_ROTARY_MOTION
        clientObject.load_type = load_type.name

        # Angular Acceleration
        clientObject.angular_acceleration = angular_acceleration

        # Angular Velocity
        clientObject.angular_velocity = angular_velocity

        # Axis Definition Type
        clientObject.axis_definition_type = axis_definition_type.name

        # Axis definition
        if axis_definition_type == MemberLoadAxisDefinitionType.AXIS_DEFINITION_TWO_POINTS.name:
            clientObject.axis_definition_p1_x = axis_definition_p1[0]
            clientObject.axis_definition_p1_y = axis_definition_p1[1]
            clientObject.axis_definition_p1_z = axis_definition_p1[2]

            clientObject.axis_definition_p2_x = axis_definition_p2[0]
            clientObject.axis_definition_p2_y = axis_definition_p2[1]
            clientObject.axis_definition_p2_z = axis_definition_p2[2]

        elif axis_definition_type == MemberLoadAxisDefinitionType.AXIS_DEFINITION_POINT_AND_AXIS.name:
            clientObject.axis_definition_p1_x = axis_definition_p1[0]
            clientObject.axis_definition_p1_y = axis_definition_p1[1]
            clientObject.axis_definition_p1_z = axis_definition_p1[2]

            clientObject.axis_definition_axis = axis_definition.name
            clientObject.axis_definition_axis_orientation = axis_orientation.name

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Load Member Load to client model
        model.clientModel.service.set_member_load(load_case_no, clientObject)


from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes
from RSTAB.enums import ModalNumberOfModes, ModalSolutionMethod, ModalMassConversionType, ModalMassMatrixType, ModalNeglectMasses

class ModalAnalysisSettings():

    def __init__(self,
                 no: int = 1,
                 name: str = 'Modal Analysis Settings',
                 number_of_modes_method = ModalNumberOfModes.NUMBER_OF_MODES_METHOD_USER_DEFINED,
                 number_of_modes : int = 4,
                 solution_method = ModalSolutionMethod.METHOD_SHIFTED_INVERSE_POWER_METHOD,
                 find_modes_beyond_frequency: bool = False,
                 frequency_f: float = 10,
                 maxmimum_natural_frequency: float = 1700,
                 effective_modal_mass_factor: float = 0.85,
                 mass_matrix_type = ModalMassMatrixType.MASS_MATRIX_TYPE_CONSISTENT,
                 mass_conversion_type = ModalMassConversionType.MASS_CONVERSION_TYPE_Z_COMPONENTS_OF_LOADS,
                 acting_masses: list = None,
                 neglect_masses = ModalNeglectMasses.E_NEGLECT_MASSES_NO_NEGLECTION,
                 comment: str = '',
                 params: dict = None,
                 model = Model):
        """
        Args:
            no (int): Setting Tag
            name (str): Setting Name
            number_of_modes_method (enum): Method for dertermining the number of nodes
            number_of_modes (int): Number of modes
            solution_method (enum): Modal solution method enumeration
            find_modes_beyond_frequency (bool): Find modes beyond frequency
            frequency_f (float): Frequency f (Hz)
            maxmimum_natural_frequency (float): Maximum natural frequency
            effective_modal_mass_factor (float): Effective modal mass factor
            mass_matrix_type (enum): Modal Mass Matrix Type Enumeration
            mass_conversion_type (enum): Modal Mass Conversion Type Enumeration
            acting_masses (list): Acting Masses Directions List
            neglect_masses (enum): Modal Neglect Masses Enumeration
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client model | Surface
        clientObject = model.clientModel.factory.create('ns0:modal_analysis_settings')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Static Analysis Settings No.
        clientObject.no = no

        # Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Analysis Type
        clientObject.solution_method = solution_method.name

        # Modes beyond frequency
        if number_of_modes_method == ModalNumberOfModes.NUMBER_OF_MODES_METHOD_USER_DEFINED:
            clientObject.number_of_modes = number_of_modes
            clientObject.find_eigenvectors_beyond_frequency = find_modes_beyond_frequency
            if find_modes_beyond_frequency:
                clientObject.frequency = frequency_f
        elif number_of_modes_method == ModalNumberOfModes.NUMBER_OF_MODES_METHOD_EFFECTIVE_MASS_FACTORS:
            clientObject.effective_modal_mass_factor = effective_modal_mass_factor
        elif number_of_modes_method == ModalNumberOfModes.NUMBER_OF_MODES_METHOD_MAXIMUM_FREQUENCY:
            # TODO: WS are missing maxmimum_natural_frequency
            #clientObject.maxmimum_natural_frequency = maxmimum_natural_frequency
            clientObject.find_eigenvectors_beyond_frequency = find_modes_beyond_frequency
            if find_modes_beyond_frequency:
                clientObject.frequency = frequency_f

        # Acting Masses
        if acting_masses:
            if len(acting_masses) == 6:
                clientObject.acting_masses_about_axis_x_enabled = acting_masses[0]
                clientObject.acting_masses_about_axis_y_enabled = acting_masses[1]
                clientObject.acting_masses_about_axis_z_enabled = acting_masses[2]
                clientObject.acting_masses_in_direction_x_enabled = acting_masses[3]
                clientObject.acting_masses_in_direction_y_enabled = acting_masses[4]
                clientObject.acting_masses_in_direction_z_enabled = acting_masses[5]
            else:
                raise ValueError('WARNING: The acting masses array needs to be of length 6. Kindly check list inputs for completeness and correctness.')

        # Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Mass Conversion Type
        clientObject.mass_conversion_type = mass_conversion_type.name

        # Mass Matrix Type
        clientObject.mass_matrix_type = mass_matrix_type.name

        # Neglect Masses
        clientObject.neglect_masses = neglect_masses.name

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Static Analysis Settings to client model
        model.clientModel.service.set_modal_analysis_settings(clientObject)

    @staticmethod
    def UserDefined(no: int = 1,
                    name: str = 'Modal Analysis User Defined',
                    number_of_modes : int = 4,
                    solution_method = ModalSolutionMethod.METHOD_SHIFTED_INVERSE_POWER_METHOD,
                    find_modes_beyond_frequency: bool = False,
                    frequency_f: float = 10,
                    mass_matrix_type = ModalMassMatrixType.MASS_MATRIX_TYPE_CONSISTENT,
                    mass_conversion_type = ModalMassConversionType.MASS_CONVERSION_TYPE_Z_COMPONENTS_OF_LOADS,
                    acting_masses: list = None,
                    neglect_masses = ModalNeglectMasses.E_NEGLECT_MASSES_NO_NEGLECTION,
                    comment: str = '',
                    params: dict = None,
                    model = Model):
        """
        Args:
            no (int): Setting Tag
            name (str): Setting Name
            number_of_modes (int): Number of modes
            solution_method (enum): Modal solution method enumeration
            find_modes_beyond_frequency (bool): Find modes beyond frequency
            frequency_f (float): Frequency f (Hz)
            mass_matrix_type (enum): Modal Mass Matrix Type Enumeration
            mass_conversion_type (enum): Modal Mass Conversion Type Enumeration
            acting_masses (list): Acting Masses Directions List
            neglect_masses (enum): Modal Neglect Masses Enumeration
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client model | Surface
        clientObject = model.clientModel.factory.create('ns0:modal_analysis_settings')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Static Analysis Settings No.
        clientObject.no = no

        # Method for Dertermining the Number of Nodes
        clientObject.number_of_modes_method = ModalNumberOfModes.NUMBER_OF_MODES_METHOD_USER_DEFINED.name

        # Analysis Type
        clientObject.solution_method = solution_method.name

        # Modes beyond frequency
        clientObject.number_of_modes = number_of_modes
        clientObject.find_eigenvectors_beyond_frequency = find_modes_beyond_frequency
        if find_modes_beyond_frequency:
            clientObject.frequency = frequency_f

        # Acting Masses
        if acting_masses and len(acting_masses) == 6:
            clientObject.acting_masses_about_axis_x_enabled = acting_masses[0]
            clientObject.acting_masses_about_axis_y_enabled = acting_masses[1]
            clientObject.acting_masses_about_axis_z_enabled = acting_masses[2]
            clientObject.acting_masses_in_direction_x_enabled = acting_masses[3]
            clientObject.acting_masses_in_direction_y_enabled = acting_masses[4]
            clientObject.acting_masses_in_direction_z_enabled = acting_masses[5]

        # Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Mass Conversion Type
        clientObject.mass_conversion_type = mass_conversion_type.name

        # Mass Matrix Type
        clientObject.mass_matrix_type = mass_matrix_type.name

        # Neglect Masses
        clientObject.neglect_masses = neglect_masses.name

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Static Analysis Settings to client model
        model.clientModel.service.set_modal_analysis_settings(clientObject)

    @staticmethod
    def EffectiveMass(no: int = 1,
                      name: str = 'Modal Analysis Effective Mass',
                      solution_method = ModalSolutionMethod.METHOD_SHIFTED_INVERSE_POWER_METHOD,
                      effective_modal_mass_factor: float = 0.85,
                      mass_matrix_type = ModalMassMatrixType.MASS_MATRIX_TYPE_CONSISTENT,
                      mass_conversion_type = ModalMassConversionType.MASS_CONVERSION_TYPE_Z_COMPONENTS_OF_LOADS,
                      acting_masses: list = None,
                      neglect_masses = ModalNeglectMasses.E_NEGLECT_MASSES_NO_NEGLECTION,
                      comment: str = '',
                      params: dict = None,
                      model = Model):
        """
        Args:
            no (int): Setting Tag
            name (str): Setting Name
            solution_method (enum): Modal solution method enumeration
            effective_modal_mass_factor (float): Effective modal mass factor
            mass_matrix_type (enum): Modal Mass Matrix Type Enumeration
            mass_conversion_type (enum): Modal Mass Conversion Type Enumeration
            acting_masses (list): Acting Masses Directions List
            neglect_masses (enum): Modal Neglect Masses Enumeration
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client model | Surface
        clientObject = model.clientModel.factory.create('ns0:modal_analysis_settings')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Static Analysis Settings No.
        clientObject.no = no

        # Method for Dertermining the Number of Nodes
        clientObject.number_of_modes_method = ModalNumberOfModes.NUMBER_OF_MODES_METHOD_EFFECTIVE_MASS_FACTORS.name

        # Analysis Type
        clientObject.solution_method = solution_method.name

        # Effective Modal Mass Factor
        clientObject.effective_modal_mass_factor = effective_modal_mass_factor

        # Acting Masses
        if acting_masses and len(acting_masses) == 6:
            clientObject.acting_masses_about_axis_x_enabled = acting_masses[0]
            clientObject.acting_masses_about_axis_y_enabled = acting_masses[1]
            clientObject.acting_masses_about_axis_z_enabled = acting_masses[2]
            clientObject.acting_masses_in_direction_x_enabled = acting_masses[3]
            clientObject.acting_masses_in_direction_y_enabled = acting_masses[4]
            clientObject.acting_masses_in_direction_z_enabled = acting_masses[5]

        # Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Mass Conversion Type
        clientObject.mass_conversion_type = mass_conversion_type.name

        # Mass Matrix Type
        clientObject.mass_matrix_type = mass_matrix_type.name

        # Neglect Masses
        clientObject.neglect_masses = neglect_masses.name

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Static Analysis Settings to client model
        model.clientModel.service.set_modal_analysis_settings(clientObject)

    @staticmethod
    def MaximumFrequency(no: int = 1,
                         name: str = 'Modal Analysis Maximum Freq',
                         solution_method = ModalSolutionMethod.METHOD_SHIFTED_INVERSE_POWER_METHOD,
                         find_modes_beyond_frequency: bool = False,
                         frequency_f: float = 10,
                         maxmimum_natural_frequency: float = 1700,
                         mass_matrix_type = ModalMassMatrixType.MASS_MATRIX_TYPE_CONSISTENT,
                         mass_conversion_type = ModalMassConversionType.MASS_CONVERSION_TYPE_Z_COMPONENTS_OF_LOADS,
                         acting_masses: list = None,
                         neglect_masses = ModalNeglectMasses.E_NEGLECT_MASSES_NO_NEGLECTION,
                         comment: str = '',
                         params: dict = None,
                         model = Model):
        """
        Args:
            no (int): Setting Tag
            name (str): Setting Name
            solution_method (enum): Modal solution method enumeration
            find_modes_beyond_frequency (bool): Find modes beyond frequency
            frequency_f (float): Frequency f (Hz)
            maxmimum_natural_frequency (float): Maximum natural frequency
            mass_matrix_type (enum): Modal Mass Matrix Type Enumeration
            mass_conversion_type (enum): Modal Mass Conversion Type Enumeration
            acting_masses (list): Acting Masses Directions List
            neglect_masses (enum): Modal Neglect Masses Enumeration
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client model | Surface
        clientObject = model.clientModel.factory.create('ns0:modal_analysis_settings')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Static Analysis Settings No.
        clientObject.no = no

        # Method for Dertermining the Number of Nodes
        clientObject.number_of_modes_method = ModalNumberOfModes.NUMBER_OF_MODES_METHOD_MAXIMUM_FREQUENCY.name

        # Analysis Type
        clientObject.solution_method = solution_method.name

        # Modes beyond frequency
        clientObject.maxmimum_natural_frequency = maxmimum_natural_frequency
        clientObject.find_eigenvectors_beyond_frequency = find_modes_beyond_frequency
        if find_modes_beyond_frequency:
            clientObject.frequency = frequency_f

        # Acting Masses
        if acting_masses and len(acting_masses) == 6:
            clientObject.acting_masses_about_axis_x_enabled = acting_masses[0]
            clientObject.acting_masses_about_axis_y_enabled = acting_masses[1]
            clientObject.acting_masses_about_axis_z_enabled = acting_masses[2]
            clientObject.acting_masses_in_direction_x_enabled = acting_masses[3]
            clientObject.acting_masses_in_direction_y_enabled = acting_masses[4]
            clientObject.acting_masses_in_direction_z_enabled = acting_masses[5]

        # Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Mass Conversion Type
        clientObject.mass_conversion_type = mass_conversion_type.name

        # Mass Matrix Type
        clientObject.mass_matrix_type = mass_matrix_type.name

        # Neglect Masses
        clientObject.neglect_masses = neglect_masses.name

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Static Analysis Settings to client model
        model.clientModel.service.set_modal_analysis_settings(clientObject)



from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes, ConvertToDlString

class MemberDefinableStiffness():
    def __init__(self,
                 no: int = 1,
                 name: str = 'MemberDefinableStiffness',
                 members: str = "1",
                 torsional_stiffness: float = 0.0,
                 bending_stiffness_y: float = 0.0,
                 bending_stiffness_z: float = 0.0,
                 axial_stiffness: float = 0.0,
                 shear_stiffness_y: float = 0.0,
                 shear_stiffness_z: float = 0.0,
                 specific_weight: float = 0.0,
                 section_area: float = 0.0,
                 rotation: float = 0.0,
                 thermal_expansion_alpha: float = 0.0,
                 thermal_expansion_width: float = 0.0,
                 thermal_expansion_height: float = 0.0,
                 comment: str = '',
                 params: dict = None,
                 model = Model):
        """
        Args:
            no (int): Member Definable Stiffness Tag
            name (str): User Defined Name
            members (str): Assigned Members
            torsional_stiffness (float): Torsional Stiffness
            bending_stiffness_y (float): Bending Stiffness in Y Direction
            bending_stiffness_z (float): Bendign Stiffness in Z Direction
            axial_stiffness (float): Axial Stiffness
            shear_stiffness_y (float): Shear Stiffness in Y Direction
            shear_stiffness_z (float): Shear Stiffness in Z Direction
            specific_weight (float): Specific Weight
            section_area (float): Section Area
            rotation (float): Rotation
            thermal_expansion_alpha (float): Thermal Expansion Alpha Coefficient
            thermal_expansion_width (float): Thermal Expansion Witdh Coefficient
            thermal_expansion_height (float): Thermal Expansion Height Coefficient
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client model | Member Definable Stffness
        clientObject = model.clientModel.factory.create('ns0:member_definable_stiffness')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Definable Stffness No.
        clientObject.no = no

        # User Defined Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Assigned Members (e.g. '5 6 7 12')
        clientObject.assigned_to = ConvertToDlString(members)

        # Torsional Stiffness
        clientObject.torsional_stiffness = torsional_stiffness

        # Bending Stiffnesses
        clientObject.bending_stiffness_y = bending_stiffness_y
        clientObject.bending_stiffness_z = bending_stiffness_z

        # Axial Stiffness
        clientObject.axial_stiffness = axial_stiffness

        # Shear Stiffnesses
        clientObject.shear_stiffness_y = shear_stiffness_y
        clientObject.shear_stiffness_z = shear_stiffness_z

        # Specific Weight
        clientObject.specific_weight = specific_weight

        # Section Area
        clientObject.section_area = section_area

        # Rotation
        clientObject.rotation = rotation

        # Thermal Expansions
        clientObject.thermal_expansion_alpha = thermal_expansion_alpha
        clientObject.thermal_expansion_width = thermal_expansion_width
        clientObject.thermal_expansion_height = thermal_expansion_height

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Member Definable Stffness to client model
        model.clientModel.service.set_member_definable_stiffness(clientObject)



from RSTAB.enums import NoteType, NoteOffsetType
from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes

class Note():

    def __init__(self,
                 no: int = 1,
                 text: str = None,
                 type = NoteType.NOTE_TYPE_POINT,
                 parameter: list = None,
                 offset_para: list = None,
                 rotation: float = 0.0,
                 display_style: int = 0,
                 name: str = None,
                 show_comment: bool = False,
                 comment: str = '',
                 params: dict = None,
                 model = Model):

        '''
        Args:
            no (int): Note Tag
            text (str): Note Text
            type (enum): Note Type Enumeration
            parameter (int or list): Note Type Parameter
                for type == NoteType.NOTE_TYPE_POINT:
                    parameter = [point_coordinate_x, point_coordinate_y, point_coordinate_z]
                for type == NoteType.NOTE_TYPE_NODE:
                    parameter = node number
                for type == NoteType.NOTE_TYPE_MEMBER:
                    parameter = [member number, note_member_reference_type enumeration, member_distance_is_defined_as_relative(bool), member_distance_relative/absolute] (if relative is true than value must be between 0.0 and 1.0)
            offset_para (list): Offset Parameter
                for offset_para[0] == NoteOffsetType.OFFSET_TYPE_XYZ:
                    offset_para = [offset_coordinate_x, offset_coordinate_y, offset_coordinate_z]
                for offset_para[0] == NoteOffsetType.OFFSET_TYPE_XY:
                    offset_para = [offset_coordinate_x, offset_coordinate_y]
                for offset_para[0] == NoteOffsetType.OFFSET_TYPE_XZ:
                    offset_para = [offset_coordinate_x, offset_coordinate_z]
                for offset_para[0] == NoteOffsetType.OFFSET_TYPE_YZ:
                    offset_para = [offset_coordinate_y, offset_coordinate_z]
            rotation (float): Note Display Rotation
            display_style (int): Note Display Style Tag (Value must be from 0 to 4)
            name (str): User Defined Name
            show_comment (bool): Enable/disable Show Comment
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Node
        clientObject = model.clientModel.factory.create('ns0:note')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Note No.
        clientObject.no = no

        # Text
        clientObject.text = text

        # Note type
        clientObject.type = type.name

        # For Note Type Point
        if type == NoteType.NOTE_TYPE_POINT:
            clientObject.point_coordinate_x = parameter[0]
            clientObject.point_coordinate_y = parameter[1]
            clientObject.point_coordinate_z = parameter[2]

        # For Note Type Node
        elif type == NoteType.NOTE_TYPE_NODE:
            clientObject.node = parameter

        # For Note Type Member
        elif type == NoteType.NOTE_TYPE_MEMBER:
            clientObject.member = parameter[0]
            clientObject.member_reference_type = parameter[1].name
            clientObject.member_distance_is_defined_as_relative = parameter[2]

            if parameter[2]:
                clientObject.member_distance_relative = parameter[3]
            else:
                clientObject.member_distance_absolute = parameter[3]

        # offset of Note

        if offset_para:
            clientObject.offset = True
            clientObject.offset_type = offset_para[0].name

            if offset_para[0] == NoteOffsetType.OFFSET_TYPE_XYZ:
                clientObject.offset_coordinate_x = offset_para[1]
                clientObject.offset_coordinate_y = offset_para[2]
                clientObject.offset_coordinate_z = offset_para[3]

            elif offset_para[0] == NoteOffsetType.OFFSET_TYPE_XY:
                clientObject.offset_coordinate_x = offset_para[1]
                clientObject.offset_coordinate_y = offset_para[2]

            elif offset_para[0] == NoteOffsetType.OFFSET_TYPE_XZ:
                clientObject.offset_coordinate_x = offset_para[1]
                clientObject.offset_coordinate_z = offset_para[2]

            elif offset_para[0] == NoteOffsetType.OFFSET_TYPE_YZ:
                clientObject.offset_coordinate_y = offset_para[1]
                clientObject.offset_coordinate_z = offset_para[2]

        clientObject.rotation = rotation

        clientObject.show_comment = show_comment
        clientObject.display_properties_index = display_style

        # Note User Defined Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Note to client model
        model.clientModel.service.set_note(clientObject)



from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes, SetAddonStatus
from RSTAB.enums import GlobalParameterUnitGroup, GlobalParameterDefinitionType, AddOn, ObjectTypes


class GlobalParameter():

    @staticmethod
    def AddParameter(
            no: int = 1,
            name: str = '',
            symbol: str = '',
            unit_group=GlobalParameterUnitGroup.LENGTH,
            definition_type=GlobalParameterDefinitionType.DEFINITION_TYPE_VALUE,
            definition_parameter: list = None,
            comment: str = '',
            params: dict = None,
            model = Model):
        '''
        Args:
            no (int): Global Parameter Tag
            name (str): Parameter Name
            symbol (str): Symbol
            unit_group (enum): Global Parameter Unit Group Enumeration
            definition_type (enum): Global Parameter Definition Type Enumeration
            definition_parameter (list): Definition Parameter List
                for definition_type = GlobalParameterDefinitionType.DEFINITION_TYPE_FORMULA:
                    definition_parameter = [formula]
                for definition_type = GlobalParameterDefinitionType.DEFINITION_TYPE_OPTIMIZATION:
                    definition_parameter = [value, min, max, steps]
                for definition_type = GlobalParameterDefinitionType.DEFINITION_TYPE_OPTIMIZATION_ASCENDING:
                    definition_parameter = [value, min, max, steps]
                for definition_type = GlobalParameterDefinitionType.DEFINITION_TYPE_OPTIMIZATION_DESCENDING:
                    definition_parameter = [value, min, max, steps]
                for definition_type = GlobalParameterDefinitionType.DEFINITION_TYPE_VALUE:
                    definition_parameter = [value]
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Global Parameter
        clientObject = model.clientModel.factory.create('ns0:global_parameter')

        # Clears object attributes | Sets all attributes to None
        clearAttributes(clientObject)

        # Global Parameter No.
        clientObject.no = no

        # Global Parameter Name
        clientObject.name = name

        # Symbol (HTML)
        clientObject.symbol = symbol

        # Unit Group
        clientObject.unit_group = unit_group.name

        # Definition Type
        clientObject.definition_type = definition_type.name

        if definition_type.name == 'DEFINITION_TYPE_FORMULA':
            if len(definition_parameter) != 1:
                raise ValueError(
                    'WARNING: The definition parameter needs to be of length 1. Kindly check list inputs for completeness and correctness.')
            clientObject.formula = definition_parameter[0]

        elif definition_type.name == 'DEFINITION_TYPE_OPTIMIZATION' or definition_type.name == 'DEFINITION_TYPE_OPTIMIZATION_ASCENDING' or definition_type.name == 'DEFINITION_TYPE_OPTIMIZATION_DESCENDING':
            SetAddonStatus(model.clientModel, AddOn.cost_estimation_active)
            if len(definition_parameter) != 4:
                raise ValueError(
                    'WARNING: The definition parameter needs to be of length 4. Kindly check list inputs for completeness and correctness.')
            clientObject.value = definition_parameter[0]
            clientObject.min = definition_parameter[1]
            clientObject.max = definition_parameter[2]
            clientObject.steps = definition_parameter[3]

        elif definition_type.name == 'DEFINITION_TYPE_VALUE':
            if len(definition_parameter) != 1:
                raise ValueError(
                    'WARNING: The definition parameter needs to be of length 1. Kindly check list inputs for completeness and correctness.')
            clientObject.value = definition_parameter[0]

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Global Parameter to client model
        model.clientModel.service.set_global_parameter(clientObject)

    @staticmethod
    def SetFormula(ObjectType = ObjectTypes.E_OBJECT_TYPE_ACTION,
                   no: int = 1,
                   parent_no: int = 1,
                   attribute: str = 'magnitude',
                   formula: str = '4 + H',
                   model = Model):
        '''
        Set formula for selected attribute of selected object.

        Args:
            ObjectType (enum): Enumeration of object type
            no (int): Id of object where formula is going to be applied
            parent_no (int): Id of parent of object - e.g. Id of load case for loads
            attribute (str): attribute of object where formula is going to be applied
            formula (str): formula
            model (RSTAB Class, optional): Model to be edited

        Returns:
            bool: True if formula was successfuly set
        '''
        objectLocation = model.clientModel.factory.create('ns0:object_location')
        objectLocation.type = ObjectType.name
        objectLocation.no = no
        objectLocation.parent_no = parent_no

        parameterAllowed = GlobalParameter.IsFormulaAllowed(ObjectType, no, parent_no, attribute)

        if parameterAllowed:
            objectParameterLocation = model.clientModel.factory.create('ns0:object_parameter_location')
            objectParameterLocation.attribute = attribute
            try:
                model.clientModel.service.set_formula(objectLocation, objectParameterLocation, formula)
                return True
            except Exception as ex:
                print(ex)
                return False
        else:
            print("Parameter not allowed")
            return False

    @staticmethod
    def GetFormula(ObjectType = ObjectTypes.E_OBJECT_TYPE_NODAL_LOAD,
                   no: int = 1,
                   parent_no: int = 1,
                   attribute: str = 'magnitude',
                   model = Model):
        '''
        Get formula for selected attribute of selected object.

        Args:
            ObjectType (enum): Enumeration of object type e.g. ObjectTypes.E_OBJECT_TYPE_NODAL_LOAD
            no (int): ID of object
            parent_no (int): Id of parent of object - e.g. Id of load case for loads
            attribute (str): attribute of object where formula is applied
            model (RSTAB Class, optional): Model to be edited

        Returns:
            dict: Formula
        '''
        objectLocation = model.clientModel.factory.create('ns0:object_location')
        objectLocation.type = ObjectType.name
        objectLocation.no = no
        objectLocation.parent_no = parent_no

        parameterAllowed = GlobalParameter.IsFormulaAllowed(ObjectType, no, parent_no, attribute)

        formula = None
        if parameterAllowed:
            objectParameterLocation = model.clientModel.factory.create('ns0:object_parameter_location')
            objectParameterLocation.attribute = attribute
            formula = model.clientModel.service.get_formula(objectLocation, objectParameterLocation)

        return formula

    @staticmethod
    def IsFormulaAllowed(ObjectType = ObjectTypes.E_OBJECT_TYPE_NODAL_LOAD,
                         no: int = 1,
                         parent_no: int = 1,
                         attribute: str = 'magnitude',
                         model = Model):
        '''
        Function check if formula can be assigned to attribute of the object.

        Args:
            ObjectType (enum): Enumeration of object type e.g. ObjectTypes.E_OBJECT_TYPE_NODAL_LOAD
            no (int): Id of object
            parent_no (int): Id of parent of object - e.g. Id of load case for loads
            attribute (str): attribute of object where formula is applied eg. 'magnitude'
            model (RSTAB Class, optional): Model to be edited

        Returns:
            bool: True if formula is allowed for attribute of the object
        '''
        parameterAllowed = False

        objectLocation = model.clientModel.factory.create('ns0:object_location')
        objectLocation.type = ObjectType.name
        objectLocation.no = no
        objectLocation.parent_no = parent_no

        allowedParameters = None

        try:
            allowedParameters = model.clientModel.service.get_list_of_parameters_formula_allowed_for(
                objectLocation)
        except Exception as ex:
            print(ex)

        if allowedParameters != None:
            for location in allowedParameters.object_parameter_location:
                if location.attribute == attribute:
                    parameterAllowed = True

        return parameterAllowed



from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes
from RSTAB.enums import AnalysisType, ActionCategoryType, ObjectTypes
from RSTAB.LoadCasesAndCombinations.loadCasesAndCombinations import LoadCasesAndCombinations
from RSTAB.LoadCasesAndCombinations.staticAnalysisSettings import StaticAnalysisSettings
from RSTAB.Tools.GetObjectNumbersByType import GetObjectNumbersByType


class LoadCase():

    def __init__(self,
                 no: int = 1,
                 name: str = 'Self-weight',
                 self_weight: list = [True, 0.0, 0.0, 1.0],
                 action_category=ActionCategoryType.ACTION_CATEGORY_NONE_NONE,
                 imperfection_case: int = None,
                 structure_modification: int = None,
                 comment: str = 'Comment',
                 params: dict = None,
                 model = Model):
        '''
        Args:
            no (int): Load Case Tag
            name (str): Load Case Name
            self_weight (list): Self-Weight Parameters
                self_weight = [self_weight_active, self_weight_factor_x, self_weight_factor_y, self_weight_factor_z]
            action_category (enum): Action Category Type Enumeration
            imperfection_case (int, optional): Assign Imperfection Case
            structure_modification (int,optional): Assign Structure Modification
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Load Case
        clientObject = model.clientModel.factory.create('ns0:load_case')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Load Case No.
        clientObject.no = no

        # Load Case Name
        clientObject.name = name

        # To Solve
        clientObject.to_solve = True

        # Analysis Type
        clientObject.analysis_type = AnalysisType.ANALYSIS_TYPE_STATIC.name
        sas = GetObjectNumbersByType(ObjectTypes.E_OBJECT_TYPE_STATIC_ANALYSIS_SETTINGS)
        if sas:
            clientObject.static_analysis_settings = sas[0]
        else:
            StaticAnalysisSettings(1)
            clientObject.static_analysis_settings = 1

        # Action Category
        if action_category.name not in LoadCasesAndCombinations.getAvailableLoadActionCategoryTypes():
            raise ValueError('WARNING: The selected Action Category is not available under the defined Standard.')
        clientObject.action_category = action_category.name

        # Self-weight Considerations
        clientObject.self_weight_active = self_weight[0]
        if not isinstance(self_weight[0], bool):
            raise ValueError('WARNING: Entry at index 0 of Self-Weight parameter to be of type bool')
        if self_weight[0]:
            if len(self_weight) != 4:
                raise ValueError('WARNING: Self-weight is activated and therefore requires a list definition of length 4. Kindly check list inputs for completeness and correctness.')
            clientObject.self_weight_factor_x = self_weight[1]
            clientObject.self_weight_factor_y = self_weight[2]
            clientObject.self_weight_factor_z = self_weight[3]
        else:
            if len(self_weight) != 1:
                raise ValueError('WARNING: Self-weight is deactivated and therefore requires a list definition of length 1. Kindly check list inputs for completeness and correctness.')

        # Options
        if imperfection_case:
            clientObject.consider_imperfection = True
            ic = GetObjectNumbersByType(ObjectTypes.E_OBJECT_TYPE_IMPERFECTION_CASE)
            if imperfection_case in ic:
                clientObject.imperfection_case = imperfection_case
        else:
            clientObject.consider_imperfection = False

        if structure_modification:
            clientObject.structure_modification_enabled = True
            sm = GetObjectNumbersByType(ObjectTypes.E_OBJECT_TYPE_STRUCTURE_MODIFICATION)
            if structure_modification in sm:
                clientObject.structure_modification = structure_modification
        else:
            clientObject.structure_modification_enabled = False

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Load Case to client model
        model.clientModel.service.set_load_case(clientObject)

    @staticmethod
    def StaticAnalysis(
            no: int = 1,
            name: str = 'Self-weight',
            to_solve: bool = True,
            analysis_settings_no: int = 1,
            action_category=ActionCategoryType.ACTION_CATEGORY_NONE_NONE,
            self_weight: list =[True, 0.0, 0.0, 10.0],
            imperfection_case: int = None,
            structure_modification: int = None,
            comment: str = 'Comment',
            params: dict = None,
            model = Model):
        '''
        Args:
            no (int): Load Case Tag
            name (str): Load Case Name
            to_solve (bool): Enable/Disbale Load Case Solver Status
            analysis_settings_no (int): Analysis Settings Number
            action_category (enum): Action Category Enumeration
            self_weight (list): Self-weight Considerations
                for self-weight considerations;
                    self_weight = [True, self_weight_factor_x, self_weight_factor_y, self_weight_factor_z]
                for no self-weight considerations;
                    self_weight = [False]
            imperfection_case (int, optional): Assign Imperfection Case
            structure_modification (int,optional): Assign Structure Modification
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Load Case
        clientObject = model.clientModel.factory.create('ns0:load_case')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Load Case No.
        clientObject.no = no

        # Load Case Name
        clientObject.name = name

        # To Solve
        clientObject.to_solve = to_solve

        # Analysis Type
        clientObject.analysis_type = AnalysisType.ANALYSIS_TYPE_STATIC.name
        sas = GetObjectNumbersByType(ObjectTypes.E_OBJECT_TYPE_STATIC_ANALYSIS_SETTINGS)
        if analysis_settings_no in sas:
            clientObject.static_analysis_settings = analysis_settings_no
        else:
            StaticAnalysisSettings(1)
            clientObject.static_analysis_settings = 1

        # Action Category
        if action_category.name not in LoadCasesAndCombinations.getAvailableLoadActionCategoryTypes():
            raise ValueError('WARNING: The selected Action Category is not available under the defined Standard.')
        clientObject.action_category = action_category.name

        # Self-weight Considerations
        clientObject.self_weight_active = self_weight[0]
        if not isinstance(self_weight[0], bool):
            raise ValueError('WARNING: Entry at index 0 of Self-Weight parameter to be of type bool')
        if self_weight[0]:
            if len(self_weight) != 4:
                raise ValueError('WARNING: Self-weight is activated and therefore requires a list definition of length 4. Kindly check list inputs for completeness and correctness.')
            clientObject.self_weight_factor_x = self_weight[1]
            clientObject.self_weight_factor_y = self_weight[2]
            clientObject.self_weight_factor_z = self_weight[3]
        else:
            if len(self_weight) != 1:
                raise ValueError('WARNING: Self-weight is deactivated and therefore requires a list definition of length 1. Kindly check list inputs for completeness and correctness.')

        # Options
        if imperfection_case:
            clientObject.consider_imperfection = True
            ic = GetObjectNumbersByType(ObjectTypes.E_OBJECT_TYPE_IMPERFECTION_CASE)
            if imperfection_case in ic:
                clientObject.imperfection_case = imperfection_case
        else:
            clientObject.consider_imperfection = False

        if structure_modification:
            clientObject.structure_modification_enabled = True
            sm = GetObjectNumbersByType(ObjectTypes.E_OBJECT_TYPE_STRUCTURE_MODIFICATION)
            if structure_modification in sm:
                clientObject.structure_modification = structure_modification
        else:
            clientObject.structure_modification_enabled = False

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Load Case to client model
        model.clientModel.service.set_load_case(clientObject)



from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes, ConvertToDlString
from RSTAB.enums import TimberServiceClassServiceClass

class TimberServiceClass():
    def __init__(self,
                no: int = 1,
                name: str = '',
                members: str = '',
                member_sets: str = '',
                service_class = TimberServiceClassServiceClass.TIMBER_SERVICE_CLASS_TYPE_1,
                comment: str = '',
                params: dict = None,
                model = Model):

        """
        Args:
            no (int): Timber Service Class Tag
            name (str): User Defined Timber Service Class Name
            members (str): Assigned Members
            member_sets (str): Assigned Member Sets
            service_class (enum): Timber Service Class Service Class
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

         # Client Model | Types For Timber Design Service Class
        clientObject = model.clientModel.factory.create('ns0:timber_service_class')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Service Class
        clientObject.no = no

        # Assigned Members
        clientObject.member = ConvertToDlString(members)

        # Assigned Member Sets
        clientObject.member_sets = ConvertToDlString(member_sets)

        # Service Class
        clientObject.service_class = service_class.name

        # User Defined Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Service Class to client model
        model.clientModel.service.set_timber_service_class(clientObject)



from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes
from RSTAB.enums import ResponseSpectrumDefinitionType

class ResponseSpectrum():

    def __init__(self,
                 no: int = 1,
                 name: str = '',
                 constant_period_step: float = None,
                 sort_table: bool = True,
                 user_defined_spectrum: list = None,
                 comment: str = '',
                 params: dict = None,
                 model = Model):
        """
        Args:
            no (int): Response Spectrum Tag
            name (str): User Defined Name
            constant_period_step (float): Enables Constant Period Step
            sort_table (bool): Sort Table Option
            user_defined_spectrum (list of lists): User Defined Spectrum
                user_defined_spectrum = [[period, acceleration], [period, acceleration], ...]
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # client model | response spectrum
        clientObject = model.clientModel.factory.create('ns0:response_spectrum')

        # clear object atributes | sets all the atributes to none
        clearAttributes(clientObject)

        # response spectrum no.
        clientObject.no = no

        # response spectrum definition type
        clientObject.definition_type = ResponseSpectrumDefinitionType.USER_DEFINED.name

        # user defined name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # constant period step option
        if constant_period_step:
            clientObject.user_defined_response_spectrum_step_enabled = True
            clientObject.user_defined_response_spectrum_period_step = constant_period_step

        # sort table option
        clientObject.user_defined_spectrum_sorted = sort_table

        # user defined spectrum
        clientObject.user_defined_response_spectrum = model.clientModel.factory.create('ns0:response_spectrum.user_defined_response_spectrum')

        if user_defined_spectrum:
            for i,j in enumerate(user_defined_spectrum):
                rsp = model.clientModel.factory.create('ns0:response_spectrum_user_defined_response_spectrum_row')
                rsp.no = i+1
                rsp.row = model.clientModel.factory.create('ns0:response_spectrum_user_defined_response_spectrum')
                clearAttributes(rsp.row)
                rsp.row.period = user_defined_spectrum[i][0]
                rsp.row.acceleration = user_defined_spectrum[i][1]

                clientObject.user_defined_response_spectrum.response_spectrum_user_defined_response_spectrum.append(rsp)

        # comment
        clientObject.comment = comment

        # adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add global parameter to client model
        model.clientModel.service.set_response_spectrum(clientObject)


    @staticmethod
    def UserDefinedGFactor(no: int = 1,
                           name: str = '',
                           constant_period_step: float = None,
                           sort_table: bool = True,
                           user_defined_spectrum: list = None,
                           comment: str = '',
                           params: dict = None,
                           model = Model):

        """
        Args:
            no (int): Response Spectrum Tag
            name (str): User Defined Name
            constant_period_step (float): Enables Constant Period Step
            sort_table (bool): Sort Table Option
            user_defined_spectrum (list): User Defined Spectrum

                user_defined_spectrum = [[period, acceleration], [period, acceleration], ...]

            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # client model | response spectrum
        clientObject = model.clientModel.factory.create('ns0:response_spectrum')

        # clear object atributes | sets all the atributes to none
        clearAttributes(clientObject)

        # response spectrum no.
        clientObject.no = no

        # response spectrum definition type
        clientObject.definition_type = ResponseSpectrumDefinitionType.USER_DEFINED_IN_G_FACTOR.name

        # user defined name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # constant period step option
        if constant_period_step:
            clientObject.user_defined_response_spectrum_step_enabled = True
            clientObject.user_defined_response_spectrum_period_step = constant_period_step

        # sort table option
        clientObject.user_defined_spectrum_sorted = sort_table

        # user defined spectrum
        clientObject.user_defined_response_spectrum = model.clientModel.factory.create('ns0:response_spectrum.user_defined_response_spectrum')

        for i,j in enumerate(user_defined_spectrum):
            rsp = model.clientModel.factory.create('ns0:response_spectrum_user_defined_response_spectrum_row')
            rsp.no = i+1
            rsp.row = model.clientModel.factory.create('ns0:response_spectrum_user_defined_response_spectrum')
            clearAttributes(rsp.row)
            rsp.row.period = user_defined_spectrum[i][0]
            rsp.row.acceleration = user_defined_spectrum[i][1]

            clientObject.user_defined_response_spectrum.response_spectrum_user_defined_response_spectrum.append(rsp)

        # comment
        clientObject.comment = comment

        # adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # add global parameter to client model
        model.clientModel.service.set_response_spectrum(clientObject)





import enum
from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes
from RSTAB.enums import OptimizerType, OptimizationTargetValueType

class OptimizationSettings():
    def __init__(self,
                 no: int = 1,
                 number_of_mutations_to_keep: int = 20,
                 target_value_type: enum = OptimizationTargetValueType.MIN_TOTAL_WEIGHT,
                 optimizer_type: enum = OptimizerType.ALL_MUTATIONS,
                 percent_of_mutations: float = 0.1,
                 optimization_values_table = None,
                 total_number_of_mutations: int = 40,
                 active: bool = True,
                 name: str = None,
                 comment: str = '',
                 params: dict = None,
                 model = Model):

        """
        Args:
            no: int = 1,
            number_of_mutations_to_keep: int = 20,
            target_value_type: enum = OptimizationTargetValueType.MIN_TOTAL_WEIGHT,
            optimizer_type: enum = OptimizerType.ALL_MUTATIONS,
            percent_of_mutations: float = 0.1,
            optimization_values_table = None,
            total_number_of_mutations: int = 40,
            active: bool = True,
            name: str = None,
            comment: str = '',
            params: dict = None,
            model = Model):
        """

        # Client model | Optimization Settings
        clientObject = model.clientModel.factory.create('ns0:optimization_settings')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Optimization Settings No.
        clientObject.no = no

        clientObject.active = active
        clientObject.number_of_mutations_to_keep = number_of_mutations_to_keep
        clientObject.target_value_type = target_value_type.name
        clientObject.optimizer_type = optimizer_type.name

        clientObject.percent_of_mutations = percent_of_mutations
        clientObject.optimization_values_table = optimization_values_table
        clientObject.total_number_of_mutations = total_number_of_mutations

        # Optimization Settings Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Set Optimization settings to client model
        model.clientModel.service.set_optimization_settings(clientObject)

    @staticmethod
    def GetOptimizationSettings(no = 1, model = Model):
        return model.clientModel.service.get_optimization_settings(no)



from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes, ConvertToDlString

class AluminumDesignULSConfigurations():

    def __init__(self,
                 no: int = 1,
                 name: str = 'ULS1',
                 members_no: str = 'All',
                 member_sets_no: str = 'All',
                 comment: str = '',
                 params: dict = None,
                 model = Model):

        """
        Args:
            no (int): Aluminum Design Ultimate Limit State Configuration Tag
            name (str): User Defined Configuration Name
            members_no (str): Assign Configuration to Selected Members
            member_sets_no (str): Assign Configuration to Selected Member Sets
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client Model | Aluminum Design Ultimate Limit State Configurations
        clientObject = model.clientModel.factory.create('ns0:aluminum_design_uls_configuration')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # ULS Configuration No.
        clientObject.no = no

        # User Defined Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Assigned Members
        if members_no == 'All':
            clientObject.assigned_to_all_members = True

        else:
            clientObject.assigned_to_all_members = False
            clientObject.assigned_to_members = ConvertToDlString(members_no)

        # Assigned Member Sets
        if member_sets_no == 'All':
            clientObject.assigned_to_all_member_sets = True

        else:
            clientObject.assigned_to_all_member_sets = False
            clientObject.assigned_to_member_sets = ConvertToDlString(member_sets_no)

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Aluminum Design ULS Configuration to Client Model
        model.clientModel.service.set_aluminum_design_uls_configuration(clientObject)



from RSTAB.initModel import Model
from RSTAB.enums import GlobalAxesOrientationType, LocalAxesOrientationType

class BaseSettings():

    def __init__(self,
                gravitational_acceleration: int = 10,
                global_axes_orientation = GlobalAxesOrientationType.E_GLOBAL_AXES_ORIENTATION_ZDOWN,
                local_axes_orientation = LocalAxesOrientationType.E_LOCAL_AXES_ORIENTATION_ZDOWN,
                tolerances: list = [0.0005, 0.0005, 0.0005, 0.0005],
                member_representatives: bool = False,
                member_set_representatives: bool = False,
                params: dict = None,
                model = Model):
        """
        Args:
            gravitational_acceleration (int): Gravitational Acceleration (m/sn2)
            global_axes_orientation (enum): Global Axes Orientation Enumeration
            local_axes_orientation (Enum): Local Axes Orientation Enumeration
            tolerances (list): Tolerances List
                tolerances = [tolerance_for_nodes, tolerance_for_lines, tolerance_for_surfaces_and_planes, tolerance_for_directions]
            member_representatives (bool): Member Representatives
            member_set_representatives (bool): Member Set Representatives
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client model | Get Model Settings
        clientObject = model.clientModel.service.get_model_settings_and_options()

        # Gravitational Acceleration
        clientObject.gravitational_acceleration = gravitational_acceleration

        # Global Axes Orientation
        clientObject.global_axes_orientation = global_axes_orientation.name

        # Local Axes Orientation
        clientObject.local_axes_orientation = local_axes_orientation.name

        # Tolerances
        if len(tolerances) != 4:
            raise ValueError("WARNING:Expected size of the array. Kindly check the list correctness.")

        clientObject.tolerance_for_nodes = tolerances[0]
        clientObject.tolerance_for_lines = tolerances[1]
        clientObject.tolerance_for_surfaces_and_planes = tolerances[2]
        clientObject.tolerance_for_directions = tolerances[3]

        # Member Representatives
        clientObject.member_representatives_active = member_representatives

        # Member Set Representatives
        clientObject.member_set_representatives_active = member_set_representatives

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Add Base Data Settings to client model
        model.clientModel.service.set_model_settings_and_options(clientObject)


class MainObjectsToActivate():

    def __init__(self,
                activate_members: bool = False,
                activate_surfaces: bool = False,
                activate_solids: bool = False,
                params: dict = None,
                model = Model):
        """
        Args:
            activate_members (bool): Activate Members
            activate_surfaces (bool): Activate Surfaces
            activate_solids (bool): Activate Solids
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client model | Get Objects To Activate
        clientObject = model.clientModel.service.get_main_objects_to_activate()

        # Activate Objects
        clientObject.activate_members = activate_members
        clientObject.activate_surfaces = activate_surfaces
        clientObject.activate_solids = activate_solids

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Add Models To Activate to client model
        model.clientModel.service.set_main_objects_to_activate(clientObject)